import os
import sys
import traceback
from typing import Any, Optional, List

from pyspark.sql.utils import AnalysisException

from spark_pipeline_framework.utilities.cannot_cast_exception_parser import (
    parse_cannot_cast_exception,
)


class FriendlySparkException(Exception):
    # noinspection PyUnusedLocal
    def __init__(
        self, exception: Exception, stage_name: Optional[str], *args: Any, **kwargs: Any
    ) -> None:
        """
        This exception wraps Spark Exceptions to extract out all the messages and show them

        :param exception:
        :param stage_name:
        :param args:
        :param kwargs:
        """
        try:
            self.exception: Exception = exception
            self.stage_name: Optional[str] = stage_name
            # Summary is a boolean argument
            # If True, it prints the exception summary
            # This way, we can avoid printing the summary all
            # the way along the exception "bubbling up"
            error_text: str = (
                (stage_name or "") + ": " + FriendlySparkException.exception_summary()
            )
            self.message: str = error_text
            if isinstance(exception, AnalysisException):
                self.message = str(exception)
            super().__init__(error_text)
        except KeyError:
            pass

    # noinspection SpellCheckingInspection
    @staticmethod
    def get_errortext(text: str) -> str:
        # Makes exception summary both BOLD and RED (FAIL)
        return text

    @staticmethod
    def exception_summary() -> str:
        # Gets the error stack
        msg = traceback.format_exc()
        try:
            # Builds the "frame" around the text
            # Gets the information about the error and makes it BOLD and RED
            info = list(
                filter(lambda t: len(t) and t[0] != "\t", msg.split("\n")[::-1])
            )
            error = FriendlySparkException.get_errortext("Error\t: {}".format(info[0]))
            # Figure out where the error happened - location (file/notebook), line and function
            idx = [t.strip()[:4] for t in info].index("File")
            where = [v.strip() for v in info[idx].strip().split(",")]
            location, line, func = where[0][5:], where[1][5:], where[2][3:]
            # If it is a pyspark error, just go with it
            if "pyspark" in error:
                new_msg = "\n{}".format(error)
            # Otherwise, build the summary
            else:
                new_msg = "\nLocation: {}\nLine\t: {}\nFunction: {}\n{}".format(
                    location, line, func, error
                )
            # now loop  through __cause__ to build up message including all the error messages
            type_, exc, tr = sys.exc_info()
            if exc:
                current_exception: BaseException = exc
                i: int = 0
                while (
                    hasattr(current_exception, "__cause__")
                    and current_exception.__cause__
                ):
                    current_exception = current_exception.__cause__
                    file_name: str = ""
                    line_number: Optional[int] = None
                    if (
                        hasattr(current_exception, "__traceback__")
                        and current_exception.__traceback__
                    ):
                        file_name = os.path.basename(
                            current_exception.__traceback__.tb_frame.f_code.co_filename
                        )
                        line_number = (
                            current_exception.__traceback__.tb_frame.f_code.co_firstlineno
                        )
                    i += 1
                    exception_message: str = str(current_exception)
                    if "cannot cast " in exception_message:
                        result: List[str] = parse_cannot_cast_exception(
                            exception_message
                        )
                        for item in result:
                            new_msg += (
                                "\n"
                                + ("<" * i)
                                + " "
                                + item
                                + " ["
                                + file_name
                                + (f" ({line_number})" if line_number else "")
                                + "]"
                            )
                    new_msg += (
                        "\n"
                        + ("<" * i)
                        + " "
                        + exception_message
                        + " ["
                        + file_name
                        + (f" ({line_number})" if line_number else "")
                        + "]"
                    )

            return new_msg
        except Exception as e:
            # If we managed to raise an exception while trying to format the original exception...
            # Oh, well...
            return "This is awkward... \n{}".format(str(e))



from pathlib import Path
from typing import Callable, Dict, Any, Union, List

from pyspark.sql import SparkSession, DataFrame
from spark_auto_mapper.automappers.automapper_base import AutoMapperBase
from spark_pipeline_framework.progress_logger.progress_logger import ProgressLogger

from spark_pipeline_framework.proxy_generator.python_transformer_helpers import (
    get_python_function_from_location,
)
from spark_pipeline_framework.transformers.framework_mapping_runner.v1.framework_mapping_runner import (
    FrameworkMappingLoader,
)


def test_can_run_framework_mapping_runner(spark_session: SparkSession) -> None:
    # Arrange
    data_dir: Path = Path(__file__).parent.parent.joinpath("./")

    spark_session.createDataFrame(
        [
            (1, "Qureshi", "Imran"),
            (2, "Vidal", "Michael"),
        ],
        ["member_id", "last_name", "first_name"],
    ).createOrReplaceTempView("patients")

    source_df: DataFrame = spark_session.table("patients")

    df = source_df.select("member_id")
    df.createOrReplaceTempView("members")

    # Act
    mapping_function: Callable[
        [Dict[str, Any]], Union[AutoMapperBase, List[AutoMapperBase]]
    ] = get_python_function_from_location(
        location=str(data_dir.joinpath("library/features/carriers_mapping/v1")),
        import_module_name=".mapping",
    )
    with ProgressLogger() as progress_logger:
        FrameworkMappingLoader(
            view="members",
            mapping_function=mapping_function,
            parameters={"foo": "bar"},
            progress_logger=progress_logger,
        ).transform(df)

    # Assert
    result_df: DataFrame = spark_session.table("members")

    result_df.printSchema()
    result_df.show(truncate=False)

    assert len(result_df.columns) == 5
    assert result_df.where("patient_id == 1").select("dst1").collect()[0][0] == "src1"
    assert (
        result_df.where("patient_id == 1").select("dst2").collect()[0][0][0]
        == "address1"
    )

    assert (
        result_df.where("patient_id == 1").select("dst3").collect()[0][0][0]
        == "address1"
    )
    assert (
        result_df.where("patient_id == 1").select("dst3").collect()[0][0][1]
        == "address2"
    )

    assert (
        result_df.where("patient_id == 1").select("dst4").collect()[0][0][0][0]
        == "usual"
    )
    assert (
        result_df.where("patient_id == 1").select("dst4").collect()[0][0][0][1]
        == "Qureshi"
    )



import inspect
from typing import List, Optional, Union, Callable

from pyspark.ml import Transformer


def run_if_else(
    enable: bool,
    stages: Union[List[Transformer], Callable[[], List[Transformer]]],
    else_stages: Optional[
        Union[List[Transformer], Callable[[], List[Transformer]]]
    ] = None,
) -> List[Transformer]:
    """
    If enable is true then it returns stages otherwise it returns else_stages
    (If stages or else_stages is a function then it evaluates it and returns the result



    :param enable: whether to return stages or else_stages
    :param stages:
    :param else_stages:
    :return:
    """
    if enable:
        if isinstance(stages, list):
            return stages
        assert inspect.isfunction(stages)
        return stages()
    else:
        if isinstance(else_stages, list):
            return else_stages
        if not else_stages:
            return []
        assert inspect.isfunction(else_stages)
        return else_stages()



from os import path, makedirs
from pathlib import Path
from shutil import rmtree

from pyspark.sql import SparkSession, DataFrame
from spark_pipeline_framework.progress_logger.progress_logger import ProgressLogger
from spark_pipeline_framework.transformers.framework_filter_by_column.v1.framework_filter_by_column_transformer import (
    FrameworkFilterByColumnTransformer,
)


def test_framework_filter_by_column_transformer(spark_session: SparkSession) -> None:
    # Arrange
    data_dir: Path = Path(__file__).parent.joinpath("./")

    temp_folder = data_dir.joinpath("./temp")
    if path.isdir(temp_folder):
        rmtree(temp_folder)
    makedirs(temp_folder)

    spark_session.createDataFrame(
        [
            (1, "Qureshi", "Imran", "1970-01-01", "female"),
            (2, "Vidal", "Michael", "1970-02-02", None),
            (3, "Paul", "Kyle", "1970-02-02", None),
        ],
        ["member_id", "last_name", "first_name", "date_of_birth", "my_gender"],
    ).createOrReplaceTempView("patients")

    source_df: DataFrame = spark_session.table("patients")

    # Act
    with ProgressLogger() as progress_logger:
        FrameworkFilterByColumnTransformer(
            column="member_id",
            include_only=[1, 3],
            view="patients",
            progress_logger=progress_logger,
        ).transform(source_df)

    # Assert
    result_df: DataFrame = spark_session.table("patients")
    result_df.show()
    assert result_df.count() == 2
    assert (
        result_df.where("member_id == 1").selectExpr("last_name").collect()[0][0]
        == "Qureshi"
    )
    assert (
        result_df.where("member_id == 3").selectExpr("last_name").collect()[0][0]
        == "Paul"
    )



from pyspark.sql.session import SparkSession

from spark_pipeline_framework.transformers.framework_db_query_runner.v1.framework_db_query_runner import (
    FrameworkDBQueryRunner,
)


def test_db_query_runner(spark_session: SparkSession) -> None:
    """
    we're only testing that the db configs are correctly exposed.
    """
    # Arrange
    username = "username"
    password = "password"
    host = "localhost"
    port = 3306
    query = "CREATE TABLE ABC(a text)"
    db = "database"

    # Act
    query_runner = FrameworkDBQueryRunner(
        username=username,
        password=password,
        host=host,
        port=port,
        db_name=db,
        query=query,
    )

    # Assert
    assert username == query_runner.getUsername()
    assert password == query_runner.getPassword()
    assert host == query_runner.getHost()
    assert port == query_runner.getPort()
    assert query == query_runner.getQuery()
    assert db == query_runner.getDb()



from typing import Optional, Any, Dict, cast

from pyspark import keyword_only
from pyspark.ml.param import Param
from pyspark.sql import DataFrame
from pyspark.sql.functions import from_json
from pyspark.sql.utils import AnalysisException
from spark_pipeline_framework.logger.yarn_logger import get_logger
from spark_pipeline_framework.progress_logger.progress_log_metric import (
    ProgressLogMetric,
)
from spark_pipeline_framework.progress_logger.progress_logger import ProgressLogger

from spark_pipeline_framework.transformers.framework_transformer.v1.framework_transformer import (
    FrameworkTransformer,
)


# move this class to SPF
class FrameworkKafkaReader(FrameworkTransformer):
    # noinspection PyUnusedLocal
    @keyword_only
    def __init__(
        self,
        kafka_brokers: str,
        topic: str,
        schema: Any,
        starting_offset: int = -2,
        use_ssl: bool = True,
        previous_checkpoint_view: Optional[str] = None,
        name: Optional[str] = None,
        parameters: Optional[Dict[str, Any]] = None,
        progress_logger: Optional[ProgressLogger] = None,
    ):
        super().__init__()

        self.logger = get_logger(__name__)

        self.kafka_brokers: Param = Param(self, "kafka_brokers", "")
        self._setDefault(kafka_brokers=kafka_brokers)

        self.topic: Param = Param(self, "topic", "")
        self._setDefault(topic=topic)

        self.schema: Param = Param(self, "schema", "")
        self._setDefault(schema=schema)

        self.starting_offset: Param = Param(self, "starting_offset", "")
        self._setDefault(starting_offset=starting_offset)

        self.use_ssl: Param = Param(self, "use_ssl", "")
        self._setDefault(use_ssl=use_ssl)

        self.previous_checkpoint_view: Param = Param(
            self, "previous_checkpoint_view", ""
        )
        self._setDefault(previous_checkpoint_view=previous_checkpoint_view)

        kwargs = self._input_kwargs
        self.setParams(**kwargs)

    # noinspection PyPep8Naming,PyMissingOrEmptyDocstring, PyUnusedLocal
    @keyword_only
    def setParams(
        self,
        kafka_brokers: str,
        topic: str,
        schema: Any,
        starting_offset: int = -2,
        use_ssl: bool = True,
        previous_checkpoint_view: Optional[str] = None,
        name: Optional[str] = None,
        parameters: Optional[Dict[str, Any]] = None,
        progress_logger: Optional[ProgressLogger] = None,
    ) -> Any:
        kwargs = self._input_kwargs
        super().setParams(
            name=name, parameters=parameters, progress_logger=progress_logger
        )
        return self._set(**kwargs)

    def _transform(self, df: DataFrame) -> DataFrame:
        kafka_brokers: str = self.getKafkaBrokers()
        topic: str = self.getTopic()
        starting_offset: int = self.getStartingOffset()
        use_ssl: bool = self.getUseSsl()
        schema: Any = self.getSchema()
        name: Optional[str] = self.getName()
        previous_checkpoint_view: Optional[str] = self.getPreviousCheckpointView()

        progress_logger: Optional[ProgressLogger] = self.getProgressLogger()

        with ProgressLogMetric(
            name=f"{name or topic}_kafka_reader", progress_logger=progress_logger
        ):
            try:
                if previous_checkpoint_view in df.sql_ctx.tableNames():
                    last_offset = (
                        df.sql_ctx.table(previous_checkpoint_view)
                        .groupBy()
                        .max("offset")
                        .collect()[0]
                        .asDict()["max(offset)"]
                    )
                    if last_offset:
                        starting_offset = last_offset + 1

                security_protocol = "PLAINTEXT"
                if use_ssl:
                    security_protocol = "SSL"

                starting_offset_text = f"""{{"{topic}":{{"0":{starting_offset}}}}}"""
                df = (
                    df.sql_ctx.read.format("kafka")
                    .option("kafka.bootstrap.servers", kafka_brokers)
                    .option("kafka.security.protocol", security_protocol)
                    .option("subscribe", f"{topic}")
                    .option("startingOffsets", starting_offset_text)
                    .load()
                )
                df = df.selectExpr("*", "CAST(value AS STRING) as event")
                df = df.withColumn("event", from_json(df.event, schema))
                df.createOrReplaceTempView(topic)
                if len(df.head(1)) == 0:
                    df.sql_ctx.table(previous_checkpoint_view).createOrReplaceTempView(
                        f"{topic}_watermark"
                    )
                else:
                    df.createOrReplaceTempView(f"{topic}_watermark")
            except AnalysisException as e:
                self.logger.error(
                    f"Failed to read from kafka topic: {topic} on {kafka_brokers}"
                )
                raise e
        return df

    # noinspection PyPep8Naming,PyMissingOrEmptyDocstring
    def getKafkaBrokers(self) -> str:
        return self.getOrDefault(self.kafka_brokers)  # type: ignore

    # noinspection PyPep8Naming,PyMissingOrEmptyDocstring
    def getTopic(self) -> str:
        return self.getOrDefault(self.topic)  # type: ignore

    # noinspection PyPep8Naming,PyMissingOrEmptyDocstring
    def getStartingOffset(self) -> int:
        return self.getOrDefault(self.starting_offset)  # type: ignore

    # noinspection PyPep8Naming,PyMissingOrEmptyDocstring
    def getUseSsl(self) -> bool:
        return self.getOrDefault(self.use_ssl)  # type: ignore

    # noinspection PyPep8Naming,PyMissingOrEmptyDocstring
    def getSchema(self) -> int:
        return self.getOrDefault(self.schema)  # type: ignore

    # noinspection PyPep8Naming,PyMissingOrEmptyDocstring
    def getPreviousCheckpointView(self) -> Optional[str]:
        return self.getOrDefault(self.previous_checkpoint_view)  # type: ignore

    # noinspection PyPep8Naming,PyMissingOrEmptyDocstring
    def getName(self) -> Optional[str]:
        return cast(str, self.getOrDefault(self.name)) or cast(
            str, self.getOrDefault(self.topic)
        )



from typing import List


def parse_cannot_cast_exception(text: str) -> List[str]:
    if "cannot cast " not in text:
        return []
    struct_comparison_text = text.split("cannot cast ")[1]
    structs = struct_comparison_text.split(" to ")
    struct1 = structs[0]
    struct2 = structs[1]
    struct2 = struct2.split(") AS")[0]
    # struct1 = 'array<struct<id:null,extension:null,code:struct<id:null,extension:null,coding:array<struct<id:null,extension:null,system:string,version:null,code:string,display:null,userSelected:null>>,text:null>,identifier:null,period:null,issuer:null>>'
    # struct1 = "array<struct<id:string,extension:array<struct<id:string,extension:array<struct<id:string,extension:array<struct<id:string>>,url:string,valueBoolean:boolean,valueCode:string,valueDate:string,valueDateTime:string,valueDecimal:int,valueId:string,valueInteger:int,valuePositiveInt:int,valueString:string,valueTime:string,valueUnsignedInt:int,valueUri:string,valueUrl:string,valueCodeableConcept:struct<id:string,extension:array<struct<id:string>>,coding:array<struct<id:string,extension:array<struct<id:string>>,system:string,version:string,code:string,display:string,userSelected:boolean>>,text:string>,valueCoding:struct<id:string,extension:array<struct<id:string>>,system:string,version:string,code:string,display:string,userSelected:boolean>,valueCount:struct<id:string,extension:array<struct<id:string>>,value:int,comparator:string,unit:string,system:string,code:string>,valueIdentifier:struct<id:string,extension:array<struct<id:string>>,use:string,type:struct<id:string,extension:array<struct<id:string>>,coding:array<struct<id:string>>,text:string>,system:string,value:string,period:struct<id:string,extension:array<struct<id:string>>,start:string,end:string>,assigner:struct<id:string,extension:array<struct<id:string>>,reference:string,type:string,identifier:struct<id:string>,display:string>>,valueMoney:struct<id:string,extension:array<struct<id:string>>,value:int,currency:string>,valuePeriod:struct<id:string,extension:array<struct<id:string>>,start:string,end:string>,valueQuantity:struct<id:string,extension:array<struct<id:string>>,value:int,comparator:string,unit:string,system:string,code:string>,valueRange:struct<id:string,extension:array<struct<id:string>>,low:struct<id:string,extension:array<struct<id:string>>,value:int,comparator:string,unit:string,system:string,code:string>,high:struct<id:string,extension:array<struct<id:string>>,value:int,comparator:string,unit:string,system:string,code:string>>,valueReference:struct<id:string,extension:array<struct<id:string>>,reference:string,type:string,identifier:struct<id:string,extension:array<struct<id:string>>,use:string,type:struct<id:string>,system:string,value:string,period:struct<id:string>,assigner:struct<id:string>>,display:string>>>,url:string,valueBoolean:boolean,valueCode:string,valueDate:string,valueDateTime:string,valueDecimal:int,valueId:string,valueInteger:int,valuePositiveInt:int,valueString:string,valueTime:string,valueUnsignedInt:int,valueUri:string,valueUrl:string,valueCodeableConcept:struct<id:string,extension:array<struct<id:string,extension:array<struct<id:string>>,url:string,valueBoolean:boolean,valueCode:string,valueDate:string,valueDateTime:string,valueDecimal:int,valueId:string,valueInteger:int,valuePositiveInt:int,valueString:string,valueTime:string,valueUnsignedInt:int,valueUri:string,valueUrl:string,valueCodeableConcept:struct<id:string,extension:array<struct<id:string>>,coding:array<struct<id:string>>,text:string>,valueCoding:struct<id:string,extension:array<struct<id:string>>,system:string,version:string,code:string,display:string,userSelected:boolean>,valueCount:struct<id:string,extension:array<struct<id:string>>,value:int,comparator:string,unit:string,system:string,code:string>,valueIdentifier:struct<id:string,extension:array<struct<id:string>>,use:string,type:struct<id:string>,system:string,value:string,period:struct<id:string>,assigner:struct<id:string>>,valueMoney:struct<id:string,extension:array<struct<id:string>>,value:int,currency:string>,valuePeriod:struct<id:string,extension:array<struct<id:string>>,start:string,end:string>,valueQuantity:struct<id:string,extension:array<struct<id:string>>,value:int,comparator:string,unit:string,system:string,code:string>,valueRange:struct<id:string,extension:array<struct<id:string>>,low:struct<id:string>,high:struct<id:string>>,valueReference:struct<id:string,extension:array<struct<id:string>>,reference:string,type:string,identifier:struct<id:string>,display:string>>>,coding:array<struct<id:string,extension:array<struct<id:string,extension:array<struct<id:string>>,url:string,valueBoolean:boolean,valueCode:string,valueDate:string,valueDateTime:string,valueDecimal:int,valueId:string,valueInteger:int,valuePositiveInt:int,valueString:string,valueTime:string,valueUnsignedInt:int,valueUri:string,valueUrl:string,valueCodeableConcept:struct<id:string>,valueCoding:struct<id:string>,valueCount:struct<id:string>,valueIdentifier:struct<id:string>,valueMoney:struct<id:string>,valuePeriod:struct<id:string>,valueQuantity:struct<id:string>,valueRange:struct<id:string>,valueReference:struct<id:string>>>,system:string,version:string,code:string,display:string,userSelected:boolean>>,text:string>,valueCoding:struct<id:string,extension:array<struct<id:string,extension:array<struct<id:string>>,url:string,valueBoolean:boolean,valueCode:string,valueDate:string,valueDateTime:string,valueDecimal:int,valueId:string,valueInteger:int,valuePositiveInt:int,valueString:string,valueTime:string,valueUnsignedInt:int,valueUri:string,valueUrl:string,valueCodeableConcept:struct<id:string,extension:array<struct<id:string>>,coding:array<struct<id:string>>,text:string>,valueCoding:struct<id:string,extension:array<struct<id:string>>,system:string,version:string,code:string,display:string,userSelected:boolean>,valueCount:struct<id:string,extension:array<struct<id:string>>,value:int,comparator:string,unit:string,system:string,code:string>,valueIdentifier:struct<id:string,extension:array<struct<id:string>>,use:string,type:struct<id:string>,system:string,value:string,period:struct<id:string>,assigner:struct<id:string>>,valueMoney:struct<id:string,extension:array<struct<id:string>>,value:int,currency:string>,valuePeriod:struct<id:string,extension:array<struct<id:string>>,start:string,end:string>,valueQuantity:struct<id:string,extension:array<struct<id:string>>,value:int,comparator:string,unit:string,system:string,code:string>,valueRange:struct<id:string,extension:array<struct<id:string>>,low:struct<id:string>,high:struct<id:string>>,valueReference:struct<id:string,extension:array<struct<id:string>>,reference:string,type:string,identifier:struct<id:string>,display:string>>>,system:string,version:string,code:string,display:string,userSelected:boolean>,valueCount:struct<id:string,extension:array<struct<id:string,extension:array<struct<id:string>>,url:string,valueBoolean:boolean,valueCode:string,valueDate:string,valueDateTime:string,valueDecimal:int,valueId:string,valueInteger:int,valuePositiveInt:int,valueString:string,valueTime:string,valueUnsignedInt:int,valueUri:string,valueUrl:string,valueCodeableConcept:struct<id:string,extension:array<struct<id:string>>,coding:array<struct<id:string>>,text:string>,valueCoding:struct<id:string,extension:array<struct<id:string>>,system:string,version:string,code:string,display:string,userSelected:boolean>,valueCount:struct<id:string,extension:array<struct<id:string>>,value:int,comparator:string,unit:string,system:string,code:string>,valueIdentifier:struct<id:string,extension:array<struct<id:string>>,use:string,type:struct<id:string>,system:string,value:string,period:struct<id:string>,assigner:struct<id:string>>,valueMoney:struct<id:string,extension:array<struct<id:string>>,value:int,currency:string>,valuePeriod:struct<id:string,extension:array<struct<id:string>>,start:string,end:string>,valueQuantity:struct<id:string,extension:array<struct<id:string>>,value:int,comparator:string,unit:string,system:string,code:string>,valueRange:struct<id:string,extension:array<struct<id:string>>,low:struct<id:string>,high:struct<id:string>>,valueReference:struct<id:string,extension:array<struct<id:string>>,reference:string,type:string,identifier:struct<id:string>,display:string>>>,value:int,comparator:string,unit:string,system:string,code:string>,valueIdentifier:struct<id:string,extension:array<struct<id:string,extension:array<struct<id:string>>,url:string,valueBoolean:boolean,valueCode:string,valueDate:string,valueDateTime:string,valueDecimal:int,valueId:string,valueInteger:int,valuePositiveInt:int,valueString:string,valueTime:string,valueUnsignedInt:int,valueUri:string,valueUrl:string,valueCodeableConcept:struct<id:string,extension:array<struct<id:string>>,coding:array<struct<id:string>>,text:string>,valueCoding:struct<id:string,extension:array<struct<id:string>>,system:string,version:string,code:string,display:string,userSelected:boolean>,valueCount:struct<id:string,extension:array<struct<id:string>>,value:int,comparator:string,unit:string,system:string,code:string>,valueIdentifier:struct<id:string,extension:array<struct<id:string>>,use:string,type:struct<id:string>,system:string,value:string,period:struct<id:string>,assigner:struct<id:string>>,valueMoney:struct<id:string,extension:array<struct<id:string>>,value:int,currency:string>,valuePeriod:struct<id:string,extension:array<struct<id:string>>,start:string,end:string>,valueQuantity:struct<id:string,extension:array<struct<id:string>>,value:int,comparator:string,unit:string,system:string,code:string>,valueRange:struct<id:string,extension:array<struct<id:string>>,low:struct<id:string>,high:struct<id:string>>,valueReference:struct<id:string,extension:array<struct<id:string>>,reference:string,type:string,identifier:struct<id:string>,display:string>>>,use:string,type:struct<id:string,extension:array<struct<id:string,extension:array<struct<id:string>>,url:string,valueBoolean:boolean,valueCode:string,valueDate:string,valueDateTime:string,valueDecimal:int,valueId:string,valueInteger:int,valuePositiveInt:int,valueString:string,valueTime:string,valueUnsignedInt:int,valueUri:string,valueUrl:string,valueCodeableConcept:struct<id:string>,valueCoding:struct<id:string>,valueCount:struct<id:string>,valueIdentifier:struct<id:string>,valueMoney:struct<id:string>,valuePeriod:struct<id:string>,valueQuantity:struct<id:string>,valueRange:struct<id:string>,valueReference:struct<id:string>>>,coding:array<struct<id:string,extension:array<struct<id:string>>,system:string,version:string,code:string,display:string,userSelected:boolean>>,text:string>,system:string,value:string,period:struct<id:string,extension:array<struct<id:string,extension:array<struct<id:string>>,url:string,valueBoolean:boolean,valueCode:string,valueDate:string,valueDateTime:string,valueDecimal:int,valueId:string,valueInteger:int,valuePositiveInt:int,valueString:string,valueTime:string,valueUnsignedInt:int,valueUri:string,valueUrl:string,valueCodeableConcept:struct<id:string>,valueCoding:struct<id:string>,valueCount:struct<id:string>,valueIdentifier:struct<id:string>,valueMoney:struct<id:string>,valuePeriod:struct<id:string>,valueQuantity:struct<id:string>,valueRange:struct<id:string>,valueReference:struct<id:string>>>,start:string,end:string>,assigner:struct<id:string,extension:array<struct<id:string,extension:array<struct<id:string>>,url:string,valueBoolean:boolean,valueCode:string,valueDate:string,valueDateTime:string,valueDecimal:int,valueId:string,valueInteger:int,valuePositiveInt:int,valueString:string,valueTime:string,valueUnsignedInt:int,valueUri:string,valueUrl:string,valueCodeableConcept:struct<id:string>,valueCoding:struct<id:string>,valueCount:struct<id:string>,valueIdentifier:struct<id:string>,valueMoney:struct<id:string>,valuePeriod:struct<id:string>,valueQuantity:struct<id:string>,valueRange:struct<id:string>,valueReference:struct<id:string>>>,reference:string,type:string,identifier:struct<id:string,extension:array<struct<id:string>>,use:string,type:struct<id:string>,system:string,value:string,period:struct<id:string>,assigner:struct<id:string>>,display:string>>,valueMoney:struct<id:string,extension:array<struct<id:string,extension:array<struct<id:string>>,url:string,valueBoolean:boolean,valueCode:string,valueDate:string,valueDateTime:string,valueDecimal:int,valueId:string,valueInteger:int,valuePositiveInt:int,valueString:string,valueTime:string,valueUnsignedInt:int,valueUri:string,valueUrl:string,valueCodeableConcept:struct<id:string,extension:array<struct<id:string>>,coding:array<struct<id:string>>,text:string>,valueCoding:struct<id:string,extension:array<struct<id:string>>,system:string,version:string,code:string,display:string,userSelected:boolean>,valueCount:struct<id:string,extension:array<struct<id:string>>,value:int,comparator:string,unit:string,system:string,code:string>,valueIdentifier:struct<id:string,extension:array<struct<id:string>>,use:string,type:struct<id:string>,system:string,value:string,period:struct<id:string>,assigner:struct<id:string>>,valueMoney:struct<id:string,extension:array<struct<id:string>>,value:int,currency:string>,valuePeriod:struct<id:string,extension:array<struct<id:string>>,start:string,end:string>,valueQuantity:struct<id:string,extension:array<struct<id:string>>,value:int,comparator:string,unit:string,system:string,code:string>,valueRange:struct<id:string,extension:array<struct<id:string>>,low:struct<id:string>,high:struct<id:string>>,valueReference:struct<id:string,extension:array<struct<id:string>>,reference:string,type:string,identifier:struct<id:string>,display:string>>>,value:int,currency:string>,valuePeriod:struct<id:string,extension:array<struct<id:string,extension:array<struct<id:string>>,url:string,valueBoolean:boolean,valueCode:string,valueDate:string,valueDateTime:string,valueDecimal:int,valueId:string,valueInteger:int,valuePositiveInt:int,valueString:string,valueTime:string,valueUnsignedInt:int,valueUri:string,valueUrl:string,valueCodeableConcept:struct<id:string,extension:array<struct<id:string>>,coding:array<struct<id:string>>,text:string>,valueCoding:struct<id:string,extension:array<struct<id:string>>,system:string,version:string,code:string,display:string,userSelected:boolean>,valueCount:struct<id:string,extension:array<struct<id:string>>,value:int,comparator:string,unit:string,system:string,code:string>,valueIdentifier:struct<id:string,extension:array<struct<id:string>>,use:string,type:struct<id:string>,system:string,value:string,period:struct<id:string>,assigner:struct<id:string>>,valueMoney:struct<id:string,extension:array<struct<id:string>>,value:int,currency:string>,valuePeriod:struct<id:string,extension:array<struct<id:string>>,start:string,end:string>,valueQuantity:struct<id:string,extension:array<struct<id:string>>,value:int,comparator:string,unit:string,system:string,code:string>,valueRange:struct<id:string,extension:array<struct<id:string>>,low:struct<id:string>,high:struct<id:string>>,valueReference:struct<id:string,extension:array<struct<id:string>>,reference:string,type:string,identifier:struct<id:string>,display:string>>>,start:string,end:string>,valueQuantity:struct<id:string,extension:array<struct<id:string,extension:array<struct<id:string>>,url:string,valueBoolean:boolean,valueCode:string,valueDate:string,valueDateTime:string,valueDecimal:int,valueId:string,valueInteger:int,valuePositiveInt:int,valueString:string,valueTime:string,valueUnsignedInt:int,valueUri:string,valueUrl:string,valueCodeableConcept:struct<id:string,extension:array<struct<id:string>>,coding:array<struct<id:string>>,text:string>,valueCoding:struct<id:string,extension:array<struct<id:string>>,system:string,version:string,code:string,display:string,userSelected:boolean>,valueCount:struct<id:string,extension:array<struct<id:string>>,value:int,comparator:string,unit:string,system:string,code:string>,valueIdentifier:struct<id:string,extension:array<struct<id:string>>,use:string,type:struct<id:string>,system:string,value:string,period:struct<id:string>,assigner:struct<id:string>>,valueMoney:struct<id:string,extension:array<struct<id:string>>,value:int,currency:string>,valuePeriod:struct<id:string,extension:array<struct<id:string>>,start:string,end:string>,valueQuantity:struct<id:string,extension:array<struct<id:string>>,value:int,comparator:string,unit:string,system:string,code:string>,valueRange:struct<id:string,extension:array<struct<id:string>>,low:struct<id:string>,high:struct<id:string>>,valueReference:struct<id:string,extension:array<struct<id:string>>,reference:string,type:string,identifier:struct<id:string>,display:string>>>,value:int,comparator:string,unit:string,system:string,code:string>,valueRange:struct<id:string,extension:array<struct<id:string,extension:array<struct<id:string>>,url:string,valueBoolean:boolean,valueCode:string,valueDate:string,valueDateTime:string,valueDecimal:int,valueId:string,valueInteger:int,valuePositiveInt:int,valueString:string,valueTime:string,valueUnsignedInt:int,valueUri:string,valueUrl:string,valueCodeableConcept:struct<id:string,extension:array<struct<id:string>>,coding:array<struct<id:string>>,text:string>,valueCoding:struct<id:string,extension:array<struct<id:string>>,system:string,version:string,code:string,display:string,userSelected:boolean>,valueCount:struct<id:string,extension:array<struct<id:string>>,value:int,comparator:string,unit:string,system:string,code:string>,valueIdentifier:struct<id:string,extension:array<struct<id:string>>,use:string,type:struct<id:string>,system:string,value:string,period:struct<id:string>,assigner:struct<id:string>>,valueMoney:struct<id:string,extension:array<struct<id:string>>,value:int,currency:string>,valuePeriod:struct<id:string,extension:array<struct<id:string>>,start:string,end:string>,valueQuantity:struct<id:string,extension:array<struct<id:string>>,value:int,comparator:string,unit:string,system:string,code:string>,valueRange:struct<id:string,extension:array<struct<id:string>>,low:struct<id:string>,high:struct<id:string>>,valueReference:struct<id:string,extension:array<struct<id:string>>,reference:string,type:string,identifier:struct<id:string>,display:string>>>,low:struct<id:string,extension:array<struct<id:string,extension:array<struct<id:string>>,url:string,valueBoolean:boolean,valueCode:string,valueDate:string,valueDateTime:string,valueDecimal:int,valueId:string,valueInteger:int,valuePositiveInt:int,valueString:string,valueTime:string,valueUnsignedInt:int,valueUri:string,valueUrl:string,valueCodeableConcept:struct<id:string>,valueCoding:struct<id:string>,valueCount:struct<id:string>,valueIdentifier:struct<id:string>,valueMoney:struct<id:string>,valuePeriod:struct<id:string>,valueQuantity:struct<id:string>,valueRange:struct<id:string>,valueReference:struct<id:string>>>,value:int,comparator:string,unit:string,system:string,code:string>,high:struct<id:string,extension:array<struct<id:string,extension:array<struct<id:string>>,url:string,valueBoolean:boolean,valueCode:string,valueDate:string,valueDateTime:string,valueDecimal:int,valueId:string,valueInteger:int,valuePositiveInt:int,valueString:string,valueTime:string,valueUnsignedInt:int,valueUri:string,valueUrl:string,valueCodeableConcept:struct<id:string>,valueCoding:struct<id:string>,valueCount:struct<id:string>,valueIdentifier:struct<id:string>,valueMoney:struct<id:string>,valuePeriod:struct<id:string>,valueQuantity:struct<id:string>,valueRange:struct<id:string>,valueReference:struct<id:string>>>,value:int,comparator:string,unit:string,system:string,code:string>>,valueReference:struct<id:string,extension:array<struct<id:string,extension:array<struct<id:string>>,url:string,valueBoolean:boolean,valueCode:string,valueDate:string,valueDateTime:string,valueDecimal:int,valueId:string,valueInteger:int,valuePositiveInt:int,valueString:string,valueTime:string,valueUnsignedInt:int,valueUri:string,valueUrl:string,valueCodeableConcept:struct<id:string,extension:array<struct<id:string>>,coding:array<struct<id:string>>,text:string>,valueCoding:struct<id:string,extension:array<struct<id:string>>,system:string,version:string,code:string,display:string,userSelected:boolean>,valueCount:struct<id:string,extension:array<struct<id:string>>,value:int,comparator:string,unit:string,system:string,code:string>,valueIdentifier:struct<id:string,extension:array<struct<id:string>>,use:string,type:struct<id:string>,system:string,value:string,period:struct<id:string>,assigner:struct<id:string>>,valueMoney:struct<id:string,extension:array<struct<id:string>>,value:int,currency:string>,valuePeriod:struct<id:string,extension:array<struct<id:string>>,start:string,end:string>,valueQuantity:struct<id:string,extension:array<struct<id:string>>,value:int,comparator:string,unit:string,system:string,code:string>,valueRange:struct<id:string,extension:array<struct<id:string>>,low:struct<id:string>,high:struct<id:string>>,valueReference:struct<id:string,extension:array<struct<id:string>>,reference:string,type:string,identifier:struct<id:string>,display:string>>>,reference:string,type:string,identifier:struct<id:string,extension:array<struct<id:string,extension:array<struct<id:string>>,url:string,valueBoolean:boolean,valueCode:string,valueDate:string,valueDateTime:string,valueDecimal:int,valueId:string,valueInteger:int,valuePositiveInt:int,valueString:string,valueTime:string,valueUnsignedInt:int,valueUri:string,valueUrl:string,valueCodeableConcept:struct<id:string>,valueCoding:struct<id:string>,valueCount:struct<id:string>,valueIdentifier:struct<id:string>,valueMoney:struct<id:string>,valuePeriod:struct<id:string>,valueQuantity:struct<id:string>,valueRange:struct<id:string>,valueReference:struct<id:string>>>,use:string,type:struct<id:string,extension:array<struct<id:string>>,coding:array<struct<id:string>>,text:string>,system:string,value:string,period:struct<id:string,extension:array<struct<id:string>>,start:string,end:string>,assigner:struct<id:string,extension:array<struct<id:string>>,reference:string,type:string,identifier:struct<id:string>,display:string>>,display:string>>>,modifierExtension:array<struct<id:string,extension:array<struct<id:string,extension:array<struct<id:string>>,url:string,valueBoolean:boolean,valueCode:string,valueDate:string,valueDateTime:string,valueDecimal:int,valueId:string,valueInteger:int,valuePositiveInt:int,valueString:string,valueTime:string,valueUnsignedInt:int,valueUri:string,valueUrl:string,valueCodeableConcept:struct<id:string,extension:array<struct<id:string>>,coding:array<struct<id:string,extension:array<struct<id:string>>,system:string,version:string,code:string,display:string,userSelected:boolean>>,text:string>,valueCoding:struct<id:string,extension:array<struct<id:string>>,system:string,version:string,code:string,display:string,userSelected:boolean>,valueCount:struct<id:string,extension:array<struct<id:string>>,value:int,comparator:string,unit:string,system:string,code:string>,valueIdentifier:struct<id:string,extension:array<struct<id:string>>,use:string,type:struct<id:string,extension:array<struct<id:string>>,coding:array<struct<id:string>>,text:string>,system:string,value:string,period:struct<id:string,extension:array<struct<id:string>>,start:string,end:string>,assigner:struct<id:string,extension:array<struct<id:string>>,reference:string,type:string,identifier:struct<id:string>,display:string>>,valueMoney:struct<id:string,extension:array<struct<id:string>>,value:int,currency:string>,valuePeriod:struct<id:string,extension:array<struct<id:string>>,start:string,end:string>,valueQuantity:struct<id:string,extension:array<struct<id:string>>,value:int,comparator:string,unit:string,system:string,code:string>,valueRange:struct<id:string,extension:array<struct<id:string>>,low:struct<id:string,extension:array<struct<id:string>>,value:int,comparator:string,unit:string,system:string,code:string>,high:struct<id:string,extension:array<struct<id:string>>,value:int,comparator:string,unit:string,system:string,code:string>>,valueReference:struct<id:string,extension:array<struct<id:string>>,reference:string,type:string,identifier:struct<id:string,extension:array<struct<id:string>>,use:string,type:struct<id:string>,system:string,value:string,period:struct<id:string>,assigner:struct<id:string>>,display:string>>>,url:string,valueBoolean:boolean,valueCode:string,valueDate:string,valueDateTime:string,valueDecimal:int,valueId:string,valueInteger:int,valuePositiveInt:int,valueString:string,valueTime:string,valueUnsignedInt:int,valueUri:string,valueUrl:string,valueCodeableConcept:struct<id:string,extension:array<struct<id:string,extension:array<struct<id:string>>,url:string,valueBoolean:boolean,valueCode:string,valueDate:string,valueDateTime:string,valueDecimal:int,valueId:string,valueInteger:int,valuePositiveInt:int,valueString:string,valueTime:string,valueUnsignedInt:int,valueUri:string,valueUrl:string,valueCodeableConcept:struct<id:string,extension:array<struct<id:string>>,coding:array<struct<id:string>>,text:string>,valueCoding:struct<id:string,extension:array<struct<id:string>>,system:string,version:string,code:string,display:string,userSelected:boolean>,valueCount:struct<id:string,extension:array<struct<id:string>>,value:int,comparator:string,unit:string,system:string,code:string>,valueIdentifier:struct<id:string,extension:array<struct<id:string>>,use:string,type:struct<id:string>,system:string,value:string,period:struct<id:string>,assigner:struct<id:string>>,valueMoney:struct<id:string,extension:array<struct<id:string>>,value:int,currency:string>,valuePeriod:struct<id:string,extension:array<struct<id:string>>,start:string,end:string>,valueQuantity:struct<id:string,extension:array<struct<id:string>>,value:int,comparator:string,unit:string,system:string,code:string>,valueRange:struct<id:string,extension:array<struct<id:string>>,low:struct<id:string>,high:struct<id:string>>,valueReference:struct<id:string,extension:array<struct<id:string>>,reference:string,type:string,identifier:struct<id:string>,display:string>>>,coding:array<struct<id:string,extension:array<struct<id:string,extension:array<struct<id:string>>,url:string,valueBoolean:boolean,valueCode:string,valueDate:string,valueDateTime:string,valueDecimal:int,valueId:string,valueInteger:int,valuePositiveInt:int,valueString:string,valueTime:string,valueUnsignedInt:int,valueUri:string,valueUrl:string,valueCodeableConcept:struct<id:string>,valueCoding:struct<id:string>,valueCount:struct<id:string>,valueIdentifier:struct<id:string>,valueMoney:struct<id:string>,valuePeriod:struct<id:string>,valueQuantity:struct<id:string>,valueRange:struct<id:string>,valueReference:struct<id:string>>>,system:string,version:string,code:string,display:string,userSelected:boolean>>,text:string>,valueCoding:struct<id:string,extension:array<struct<id:string,extension:array<struct<id:string>>,url:string,valueBoolean:boolean,valueCode:string,valueDate:string,valueDateTime:string,valueDecimal:int,valueId:string,valueInteger:int,valuePositiveInt:int,valueString:string,valueTime:string,valueUnsignedInt:int,valueUri:string,valueUrl:string,valueCodeableConcept:struct<id:string,extension:array<struct<id:string>>,coding:array<struct<id:string>>,text:string>,valueCoding:struct<id:string,extension:array<struct<id:string>>,system:string,version:string,code:string,display:string,userSelected:boolean>,valueCount:struct<id:string,extension:array<struct<id:string>>,value:int,comparator:string,unit:string,system:string,code:string>,valueIdentifier:struct<id:string,extension:array<struct<id:string>>,use:string,type:struct<id:string>,system:string,value:string,period:struct<id:string>,assigner:struct<id:string>>,valueMoney:struct<id:string,extension:array<struct<id:string>>,value:int,currency:string>,valuePeriod:struct<id:string,extension:array<struct<id:string>>,start:string,end:string>,valueQuantity:struct<id:string,extension:array<struct<id:string>>,value:int,comparator:string,unit:string,system:string,code:string>,valueRange:struct<id:string,extension:array<struct<id:string>>,low:struct<id:string>,high:struct<id:string>>,valueReference:struct<id:string,extension:array<struct<id:string>>,reference:string,type:string,identifier:struct<id:string>,display:string>>>,system:string,version:string,code:string,display:string,userSelected:boolean>,valueCount:struct<id:string,extension:array<struct<id:string,extension:array<struct<id:string>>,url:string,valueBoolean:boolean,valueCode:string,valueDate:string,valueDateTime:string,valueDecimal:int,valueId:string,valueInteger:int,valuePositiveInt:int,valueString:string,valueTime:string,valueUnsignedInt:int,valueUri:string,valueUrl:string,valueCodeableConcept:struct<id:string,extension:array<struct<id:string>>,coding:array<struct<id:string>>,text:string>,valueCoding:struct<id:string,extension:array<struct<id:string>>,system:string,version:string,code:string,display:string,userSelected:boolean>,valueCount:struct<id:string,extension:array<struct<id:string>>,value:int,comparator:string,unit:string,system:string,code:string>,valueIdentifier:struct<id:string,extension:array<struct<id:string>>,use:string,type:struct<id:string>,system:string,value:string,period:struct<id:string>,assigner:struct<id:string>>,valueMoney:struct<id:string,extension:array<struct<id:string>>,value:int,currency:string>,valuePeriod:struct<id:string,extension:array<struct<id:string>>,start:string,end:string>,valueQuantity:struct<id:string,extension:array<struct<id:string>>,value:int,comparator:string,unit:string,system:string,code:string>,valueRange:struct<id:string,extension:array<struct<id:string>>,low:struct<id:string>,high:struct<id:string>>,valueReference:struct<id:string,extension:array<struct<id:string>>,reference:string,type:string,identifier:struct<id:string>,display:string>>>,value:int,comparator:string,unit:string,system:string,code:string>,valueIdentifier:struct<id:string,extension:array<struct<id:string,extension:array<struct<id:string>>,url:string,valueBoolean:boolean,valueCode:string,valueDate:string,valueDateTime:string,valueDecimal:int,valueId:string,valueInteger:int,valuePositiveInt:int,valueString:string,valueTime:string,valueUnsignedInt:int,valueUri:string,valueUrl:string,valueCodeableConcept:struct<id:string,extension:array<struct<id:string>>,coding:array<struct<id:string>>,text:string>,valueCoding:struct<id:string,extension:array<struct<id:string>>,system:string,version:string,code:string,display:string,userSelected:boolean>,valueCount:struct<id:string,extension:array<struct<id:string>>,value:int,comparator:string,unit:string,system:string,code:string>,valueIdentifier:struct<id:string,extension:array<struct<id:string>>,use:string,type:struct<id:string>,system:string,value:string,period:struct<id:string>,assigner:struct<id:string>>,valueMoney:struct<id:string,extension:array<struct<id:string>>,value:int,currency:string>,valuePeriod:struct<id:string,extension:array<struct<id:string>>,start:string,end:string>,valueQuantity:struct<id:string,extension:array<struct<id:string>>,value:int,comparator:string,unit:string,system:string,code:string>,valueRange:struct<id:string,extension:array<struct<id:string>>,low:struct<id:string>,high:struct<id:string>>,valueReference:struct<id:string,extension:array<struct<id:string>>,reference:string,type:string,identifier:struct<id:string>,display:string>>>,use:string,type:struct<id:string,extension:array<struct<id:string,extension:array<struct<id:string>>,url:string,valueBoolean:boolean,valueCode:string,valueDate:string,valueDateTime:string,valueDecimal:int,valueId:string,valueInteger:int,valuePositiveInt:int,valueString:string,valueTime:string,valueUnsignedInt:int,valueUri:string,valueUrl:string,valueCodeableConcept:struct<id:string>,valueCoding:struct<id:string>,valueCount:struct<id:string>,valueIdentifier:struct<id:string>,valueMoney:struct<id:string>,valuePeriod:struct<id:string>,valueQuantity:struct<id:string>,valueRange:struct<id:string>,valueReference:struct<id:string>>>,coding:array<struct<id:string,extension:array<struct<id:string>>,system:string,version:string,code:string,display:string,userSelected:boolean>>,text:string>,system:string,value:string,period:struct<id:string,extension:array<struct<id:string,extension:array<struct<id:string>>,url:string,valueBoolean:boolean,valueCode:string,valueDate:string,valueDateTime:string,valueDecimal:int,valueId:string,valueInteger:int,valuePositiveInt:int,valueString:string,valueTime:string,valueUnsignedInt:int,valueUri:string,valueUrl:string,valueCodeableConcept:struct<id:string>,valueCoding:struct<id:string>,valueCount:struct<id:string>,valueIdentifier:struct<id:string>,valueMoney:struct<id:string>,valuePeriod:struct<id:string>,valueQuantity:struct<id:string>,valueRange:struct<id:string>,valueReference:struct<id:string>>>,start:string,end:string>,assigner:struct<id:string,extension:array<struct<id:string,extension:array<struct<id:string>>,url:string,valueBoolean:boolean,valueCode:string,valueDate:string,valueDateTime:string,valueDecimal:int,valueId:string,valueInteger:int,valuePositiveInt:int,valueString:string,valueTime:string,valueUnsignedInt:int,valueUri:string,valueUrl:string,valueCodeableConcept:struct<id:string>,valueCoding:struct<id:string>,valueCount:struct<id:string>,valueIdentifier:struct<id:string>,valueMoney:struct<id:string>,valuePeriod:struct<id:string>,valueQuantity:struct<id:string>,valueRange:struct<id:string>,valueReference:struct<id:string>>>,reference:string,type:string,identifier:struct<id:string,extension:array<struct<id:string>>,use:string,type:struct<id:string>,system:string,value:string,period:struct<id:string>,assigner:struct<id:string>>,display:string>>,valueMoney:struct<id:string,extension:array<struct<id:string,extension:array<struct<id:string>>,url:string,valueBoolean:boolean,valueCode:string,valueDate:string,valueDateTime:string,valueDecimal:int,valueId:string,valueInteger:int,valuePositiveInt:int,valueString:string,valueTime:string,valueUnsignedInt:int,valueUri:string,valueUrl:string,valueCodeableConcept:struct<id:string,extension:array<struct<id:string>>,coding:array<struct<id:string>>,text:string>,valueCoding:struct<id:string,extension:array<struct<id:string>>,system:string,version:string,code:string,display:string,userSelected:boolean>,valueCount:struct<id:string,extension:array<struct<id:string>>,value:int,comparator:string,unit:string,system:string,code:string>,valueIdentifier:struct<id:string,extension:array<struct<id:string>>,use:string,type:struct<id:string>,system:string,value:string,period:struct<id:string>,assigner:struct<id:string>>,valueMoney:struct<id:string,extension:array<struct<id:string>>,value:int,currency:string>,valuePeriod:struct<id:string,extension:array<struct<id:string>>,start:string,end:string>,valueQuantity:struct<id:string,extension:array<struct<id:string>>,value:int,comparator:string,unit:string,system:string,code:string>,valueRange:struct<id:string,extension:array<struct<id:string>>,low:struct<id:string>,high:struct<id:string>>,valueReference:struct<id:string,extension:array<struct<id:string>>,reference:string,type:string,identifier:struct<id:string>,display:string>>>,value:int,currency:string>,valuePeriod:struct<id:string,extension:array<struct<id:string,extension:array<struct<id:string>>,url:string,valueBoolean:boolean,valueCode:string,valueDate:string,valueDateTime:string,valueDecimal:int,valueId:string,valueInteger:int,valuePositiveInt:int,valueString:string,valueTime:string,valueUnsignedInt:int,valueUri:string,valueUrl:string,valueCodeableConcept:struct<id:string,extension:array<struct<id:string>>,coding:array<struct<id:string>>,text:string>,valueCoding:struct<id:string,extension:array<struct<id:string>>,system:string,version:string,code:string,display:string,userSelected:boolean>,valueCount:struct<id:string,extension:array<struct<id:string>>,value:int,comparator:string,unit:string,system:string,code:string>,valueIdentifier:struct<id:string,extension:array<struct<id:string>>,use:string,type:struct<id:string>,system:string,value:string,period:struct<id:string>,assigner:struct<id:string>>,valueMoney:struct<id:string,extension:array<struct<id:string>>,value:int,currency:string>,valuePeriod:struct<id:string,extension:array<struct<id:string>>,start:string,end:string>,valueQuantity:struct<id:string,extension:array<struct<id:string>>,value:int,comparator:string,unit:string,system:string,code:string>,valueRange:struct<id:string,extension:array<struct<id:string>>,low:struct<id:string>,high:struct<id:string>>,valueReference:struct<id:string,extension:array<struct<id:string>>,reference:string,type:string,identifier:struct<id:string>,display:string>>>,start:string,end:string>,valueQuantity:struct<id:string,extension:array<struct<id:string,extension:array<struct<id:string>>,url:string,valueBoolean:boolean,valueCode:string,valueDate:string,valueDateTime:string,valueDecimal:int,valueId:string,valueInteger:int,valuePositiveInt:int,valueString:string,valueTime:string,valueUnsignedInt:int,valueUri:string,valueUrl:string,valueCodeableConcept:struct<id:string,extension:array<struct<id:string>>,coding:array<struct<id:string>>,text:string>,valueCoding:struct<id:string,extension:array<struct<id:string>>,system:string,version:string,code:string,display:string,userSelected:boolean>,valueCount:struct<id:string,extension:array<struct<id:string>>,value:int,comparator:string,unit:string,system:string,code:string>,valueIdentifier:struct<id:string,extension:array<struct<id:string>>,use:string,type:struct<id:string>,system:string,value:string,period:struct<id:string>,assigner:struct<id:string>>,valueMoney:struct<id:string,extension:array<struct<id:string>>,value:int,currency:string>,valuePeriod:struct<id:string,extension:array<struct<id:string>>,start:string,end:string>,valueQuantity:struct<id:string,extension:array<struct<id:string>>,value:int,comparator:string,unit:string,system:string,code:string>,valueRange:struct<id:string,extension:array<struct<id:string>>,low:struct<id:string>,high:struct<id:string>>,valueReference:struct<id:string,extension:array<struct<id:string>>,reference:string,type:string,identifier:struct<id:string>,display:string>>>,value:int,comparator:string,unit:string,system:string,code:string>,valueRange:struct<id:string,extension:array<struct<id:string,extension:array<struct<id:string>>,url:string,valueBoolean:boolean,valueCode:string,valueDate:string,valueDateTime:string,valueDecimal:int,valueId:string,valueInteger:int,valuePositiveInt:int,valueString:string,valueTime:string,valueUnsignedInt:int,valueUri:string,valueUrl:string,valueCodeableConcept:struct<id:string,extension:array<struct<id:string>>,coding:array<struct<id:string>>,text:string>,valueCoding:struct<id:string,extension:array<struct<id:string>>,system:string,version:string,code:string,display:string,userSelected:boolean>,valueCount:struct<id:string,extension:array<struct<id:string>>,value:int,comparator:string,unit:string,system:string,code:string>,valueIdentifier:struct<id:string,extension:array<struct<id:string>>,use:string,type:struct<id:string>,system:string,value:string,period:struct<id:string>,assigner:struct<id:string>>,valueMoney:struct<id:string,extension:array<struct<id:string>>,value:int,currency:string>,valuePeriod:struct<id:string,extension:array<struct<id:string>>,start:string,end:string>,valueQuantity:struct<id:string,extension:array<struct<id:string>>,value:int,comparator:string,unit:string,system:string,code:string>,valueRange:struct<id:string,extension:array<struct<id:string>>,low:struct<id:string>,high:struct<id:string>>,valueReference:struct<id:string,extension:array<struct<id:string>>,reference:string,type:string,identifier:struct<id:string>,display:string>>>,low:struct<id:string,extension:array<struct<id:string,extension:array<struct<id:string>>,url:string,valueBoolean:boolean,valueCode:string,valueDate:string,valueDateTime:string,valueDecimal:int,valueId:string,valueInteger:int,valuePositiveInt:int,valueString:string,valueTime:string,valueUnsignedInt:int,valueUri:string,valueUrl:string,valueCodeableConcept:struct<id:string>,valueCoding:struct<id:string>,valueCount:struct<id:string>,valueIdentifier:struct<id:string>,valueMoney:struct<id:string>,valuePeriod:struct<id:string>,valueQuantity:struct<id:string>,valueRange:struct<id:string>,valueReference:struct<id:string>>>,value:int,comparator:string,unit:string,system:string,code:string>,high:struct<id:string,extension:array<struct<id:string,extension:array<struct<id:string>>,url:string,valueBoolean:boolean,valueCode:string,valueDate:string,valueDateTime:string,valueDecimal:int,valueId:string,valueInteger:int,valuePositiveInt:int,valueString:string,valueTime:string,valueUnsignedInt:int,valueUri:string,valueUrl:string,valueCodeableConcept:struct<id:string>,valueCoding:struct<id:string>,valueCount:struct<id:string>,valueIdentifier:struct<id:string>,valueMoney:struct<id:string>,valuePeriod:struct<id:string>,valueQuantity:struct<id:string>,valueRange:struct<id:string>,valueReference:struct<id:string>>>,value:int,comparator:string,unit:string,system:string,code:string>>,valueReference:struct<id:string,extension:array<struct<id:string,extension:array<struct<id:string>>,url:string,valueBoolean:boolean,valueCode:string,valueDate:string,valueDateTime:string,valueDecimal:int,valueId:string,valueInteger:int,valuePositiveInt:int,valueString:string,valueTime:string,valueUnsignedInt:int,valueUri:string,valueUrl:string,valueCodeableConcept:struct<id:string,extension:array<struct<id:string>>,coding:array<struct<id:string>>,text:string>,valueCoding:struct<id:string,extension:array<struct<id:string>>,system:string,version:string,code:string,display:string,userSelected:boolean>,valueCount:struct<id:string,extension:array<struct<id:string>>,value:int,comparator:string,unit:string,system:string,code:string>,valueIdentifier:struct<id:string,extension:array<struct<id:string>>,use:string,type:struct<id:string>,system:string,value:string,period:struct<id:string>,assigner:struct<id:string>>,valueMoney:struct<id:string,extension:array<struct<id:string>>,value:int,currency:string>,valuePeriod:struct<id:string,extension:array<struct<id:string>>,start:string,end:string>,valueQuantity:struct<id:string,extension:array<struct<id:string>>,value:int,comparator:string,unit:string,system:string,code:string>,valueRange:struct<id:string,extension:array<struct<id:string>>,low:struct<id:string>,high:struct<id:string>>,valueReference:struct<id:string,extension:array<struct<id:string>>,reference:string,type:string,identifier:struct<id:string>,display:string>>>,reference:string,type:string,identifier:struct<id:string,extension:array<struct<id:string,extension:array<struct<id:string>>,url:string,valueBoolean:boolean,valueCode:string,valueDate:string,valueDateTime:string,valueDecimal:int,valueId:string,valueInteger:int,valuePositiveInt:int,valueString:string,valueTime:string,valueUnsignedInt:int,valueUri:string,valueUrl:string,valueCodeableConcept:struct<id:string>,valueCoding:struct<id:string>,valueCount:struct<id:string>,valueIdentifier:struct<id:string>,valueMoney:struct<id:string>,valuePeriod:struct<id:string>,valueQuantity:struct<id:string>,valueRange:struct<id:string>,valueReference:struct<id:string>>>,use:string,type:struct<id:string,extension:array<struct<id:string>>,coding:array<struct<id:string>>,text:string>,system:string,value:string,period:struct<id:string,extension:array<struct<id:string>>,start:string,end:string>,assigner:struct<id:string,extension:array<struct<id:string>>,reference:string,type:string,identifier:struct<id:string>,display:string>>,display:string>>>,identifier:array<struct<id:string,extension:array<struct<id:string,extension:array<struct<id:string,extension:array<struct<id:string>>,url:string,valueBoolean:boolean,valueCode:string,valueDate:string,valueDateTime:string,valueDecimal:int,valueId:string,valueInteger:int,valuePositiveInt:int,valueString:string,valueTime:string,valueUnsignedInt:int,valueUri:string,valueUrl:string,valueCodeableConcept:struct<id:string,extension:array<struct<id:string>>,coding:array<struct<id:string>>,text:string>,valueCoding:struct<id:string,extension:array<struct<id:string>>,system:string,version:string,code:string,display:string,userSelected:boolean>,valueCount:struct<id:string,extension:array<struct<id:string>>,value:int,comparator:string,unit:string,system:string,code:string>,valueIdentifier:struct<id:string,extension:array<struct<id:string>>,use:string,type:struct<id:string>,system:string,value:string,period:struct<id:string>,assigner:struct<id:string>>,valueMoney:struct<id:string,extension:array<struct<id:string>>,value:int,currency:string>,valuePeriod:struct<id:string,extension:array<struct<id:string>>,start:string,end:string>,valueQuantity:struct<id:string,extension:array<struct<id:string>>,value:int,comparator:string,unit:string,system:string,code:string>,valueRange:struct<id:string,extension:array<struct<id:string>>,low:struct<id:string>,high:struct<id:string>>,valueReference:struct<id:string,extension:array<struct<id:string>>,reference:string,type:string,identifier:struct<id:string>,display:string>>>,url:string,valueBoolean:boolean,valueCode:string,valueDate:string,valueDateTime:string,valueDecimal:int,valueId:string,valueInteger:int,valuePositiveInt:int,valueString:string,valueTime:string,valueUnsignedInt:int,valueUri:string,valueUrl:string,valueCodeableConcept:struct<id:string,extension:array<struct<id:string,extension:array<struct<id:string>>,url:string,valueBoolean:boolean,valueCode:string,valueDate:string,valueDateTime:string,valueDecimal:int,valueId:string,valueInteger:int,valuePositiveInt:int,valueString:string,valueTime:string,valueUnsignedInt:int,valueUri:string,valueUrl:string,valueCodeableConcept:struct<id:string>,valueCoding:struct<id:string>,valueCount:struct<id:string>,valueIdentifier:struct<id:string>,valueMoney:struct<id:string>,valuePeriod:struct<id:string>,valueQuantity:struct<id:string>,valueRange:struct<id:string>,valueReference:struct<id:string>>>,coding:array<struct<id:string,extension:array<struct<id:string>>,system:string,version:string,code:string,display:string,userSelected:boolean>>,text:string>,valueCoding:struct<id:string,extension:array<struct<id:string,extension:array<struct<id:string>>,url:string,valueBoolean:boolean,valueCode:string,valueDate:string,valueDateTime:string,valueDecimal:int,valueId:string,valueInteger:int,valuePositiveInt:int,valueString:string,valueTime:string,valueUnsignedInt:int,valueUri:string,valueUrl:string,valueCodeableConcept:struct<id:string>,valueCoding:struct<id:string>,valueCount:struct<id:string>,valueIdentifier:struct<id:string>,valueMoney:struct<id:string>,valuePeriod:struct<id:string>,valueQuantity:struct<id:string>,valueRange:struct<id:string>,valueReference:struct<id:string>>>,system:string,version:string,code:string,display:string,userSelected:boolean>,valueCount:struct<id:string,extension:array<struct<id:string,extension:array<struct<id:string>>,url:string,valueBoolean:boolean,valueCode:string,valueDate:string,valueDateTime:string,valueDecimal:int,valueId:string,valueInteger:int,valuePositiveInt:int,valueString:string,valueTime:string,valueUnsignedInt:int,valueUri:string,valueUrl:string,valueCodeableConcept:struct<id:string>,valueCoding:struct<id:string>,valueCount:struct<id:string>,valueIdentifier:struct<id:string>,valueMoney:struct<id:string>,valuePeriod:struct<id:string>,valueQuantity:struct<id:string>,valueRange:struct<id:string>,valueReference:struct<id:string>>>,value:int,comparator:string,unit:string,system:string,code:string>,valueIdentifier:struct<id:string,extension:array<struct<id:string,extension:array<struct<id:string>>,url:string,valueBoolean:boolean,valueCode:string,valueDate:string,valueDateTime:string,valueDecimal:int,valueId:string,valueInteger:int,valuePositiveInt:int,valueString:string,valueTime:string,valueUnsignedInt:int,valueUri:string,valueUrl:string,valueCodeableConcept:struct<id:string>,valueCoding:struct<id:string>,valueCount:struct<id:string>,valueIdentifier:struct<id:string>,valueMoney:struct<id:string>,valuePeriod:struct<id:string>,valueQuantity:struct<id:string>,valueRange:struct<id:string>,valueReference:struct<id:string>>>,use:string,type:struct<id:string,extension:array<struct<id:string>>,coding:array<struct<id:string>>,text:string>,system:string,value:string,period:struct<id:string,extension:array<struct<id:string>>,start:string,end:string>,assigner:struct<id:string,extension:array<struct<id:string>>,reference:string,type:string,identifier:struct<id:string>,display:string>>,valueMoney:struct<id:string,extension:array<struct<id:string,extension:array<struct<id:string>>,url:string,valueBoolean:boolean,valueCode:string,valueDate:string,valueDateTime:string,valueDecimal:int,valueId:string,valueInteger:int,valuePositiveInt:int,valueString:string,valueTime:string,valueUnsignedInt:int,valueUri:string,valueUrl:string,valueCodeableConcept:struct<id:string>,valueCoding:struct<id:string>,valueCount:struct<id:string>,valueIdentifier:struct<id:string>,valueMoney:struct<id:string>,valuePeriod:struct<id:string>,valueQuantity:struct<id:string>,valueRange:struct<id:string>,valueReference:struct<id:string>>>,value:int,currency:string>,valuePeriod:struct<id:string,extension:array<struct<id:string,extension:array<struct<id:string>>,url:string,valueBoolean:boolean,valueCode:string,valueDate:string,valueDateTime:string,valueDecimal:int,valueId:string,valueInteger:int,valuePositiveInt:int,valueString:string,valueTime:string,valueUnsignedInt:int,valueUri:string,valueUrl:string,valueCodeableConcept:struct<id:string>,valueCoding:struct<id:string>,valueCount:struct<id:string>,valueIdentifier:struct<id:string>,valueMoney:struct<id:string>,valuePeriod:struct<id:string>,valueQuantity:struct<id:string>,valueRange:struct<id:string>,valueReference:struct<id:string>>>,start:string,end:string>,valueQuantity:struct<id:string,extension:array<struct<id:string,extension:array<struct<id:string>>,url:string,valueBoolean:boolean,valueCode:string,valueDate:string,valueDateTime:string,valueDecimal:int,valueId:string,valueInteger:int,valuePositiveInt:int,valueString:string,valueTime:string,valueUnsignedInt:int,valueUri:string,valueUrl:string,valueCodeableConcept:struct<id:string>,valueCoding:struct<id:string>,valueCount:struct<id:string>,valueIdentifier:struct<id:string>,valueMoney:struct<id:string>,valuePeriod:struct<id:string>,valueQuantity:struct<id:string>,valueRange:struct<id:string>,valueReference:struct<id:string>>>,value:int,comparator:string,unit:string,system:string,code:string>,valueRange:struct<id:string,extension:array<struct<id:string,extension:array<struct<id:string>>,url:string,valueBoolean:boolean,valueCode:string,valueDate:string,valueDateTime:string,valueDecimal:int,valueId:string,valueInteger:int,valuePositiveInt:int,valueString:string,valueTime:string,valueUnsignedInt:int,valueUri:string,valueUrl:string,valueCodeableConcept:struct<id:string>,valueCoding:struct<id:string>,valueCount:struct<id:string>,valueIdentifier:struct<id:string>,valueMoney:struct<id:string>,valuePeriod:struct<id:string>,valueQuantity:struct<id:string>,valueRange:struct<id:string>,valueReference:struct<id:string>>>,low:struct<id:string,extension:array<struct<id:string>>,value:int,comparator:string,unit:string,system:string,code:string>,high:struct<id:string,extension:array<struct<id:string>>,value:int,comparator:string,unit:string,system:string,code:string>>,valueReference:struct<id:string,extension:array<struct<id:string,extension:array<struct<id:string>>,url:string,valueBoolean:boolean,valueCode:string,valueDate:string,valueDateTime:string,valueDecimal:int,valueId:string,valueInteger:int,valuePositiveInt:int,valueString:string,valueTime:string,valueUnsignedInt:int,valueUri:string,valueUrl:string,valueCodeableConcept:struct<id:string>,valueCoding:struct<id:string>,valueCount:struct<id:string>,valueIdentifier:struct<id:string>,valueMoney:struct<id:string>,valuePeriod:struct<id:string>,valueQuantity:struct<id:string>,valueRange:struct<id:string>,valueReference:struct<id:string>>>,reference:string,type:string,identifier:struct<id:string,extension:array<struct<id:string>>,use:string,type:struct<id:string>,system:string,value:string,period:struct<id:string>,assigner:struct<id:string>>,display:string>>>,use:string,type:struct<id:string,extension:array<struct<id:string,extension:array<struct<id:string,extension:array<struct<id:string>>,url:string,valueBoolean:boolean,valueCode:string,valueDate:string,valueDateTime:string,valueDecimal:int,valueId:string,valueInteger:int,valuePositiveInt:int,valueString:string,valueTime:string,valueUnsignedInt:int,valueUri:string,valueUrl:string,valueCodeableConcept:struct<id:string>,valueCoding:struct<id:string>,valueCount:struct<id:string>,valueIdentifier:struct<id:string>,valueMoney:struct<id:string>,valuePeriod:struct<id:string>,valueQuantity:struct<id:string>,valueRange:struct<id:string>,valueReference:struct<id:string>>>,url:string,valueBoolean:boolean,valueCode:string,valueDate:string,valueDateTime:string,valueDecimal:int,valueId:string,valueInteger:int,valuePositiveInt:int,valueString:string,valueTime:string,valueUnsignedInt:int,valueUri:string,valueUrl:string,valueCodeableConcept:struct<id:string,extension:array<struct<id:string>>,coding:array<struct<id:string>>,text:string>,valueCoding:struct<id:string,extension:array<struct<id:string>>,system:string,version:string,code:string,display:string,userSelected:boolean>,valueCount:struct<id:string,extension:array<struct<id:string>>,value:int,comparator:string,unit:string,system:string,code:string>,valueIdentifier:struct<id:string,extension:array<struct<id:string>>,use:string,type:struct<id:string>,system:string,value:string,period:struct<id:string>,assigner:struct<id:string>>,valueMoney:struct<id:string,extension:array<struct<id:string>>,value:int,currency:string>,valuePeriod:struct<id:string,extension:array<struct<id:string>>,start:string,end:string>,valueQuantity:struct<id:string,extension:array<struct<id:string>>,value:int,comparator:string,unit:string,system:string,code:string>,valueRange:struct<id:string,extension:array<struct<id:string>>,low:struct<id:string>,high:struct<id:string>>,valueReference:struct<id:string,extension:array<struct<id:string>>,reference:string,type:string,identifier:struct<id:string>,display:string>>>,coding:array<struct<id:string,extension:array<struct<id:string,extension:array<struct<id:string>>,url:string,valueBoolean:boolean,valueCode:string,valueDate:string,valueDateTime:string,valueDecimal:int,valueId:string,valueInteger:int,valuePositiveInt:int,valueString:string,valueTime:string,valueUnsignedInt:int,valueUri:string,valueUrl:string,valueCodeableConcept:struct<id:string>,valueCoding:struct<id:string>,valueCount:struct<id:string>,valueIdentifier:struct<id:string>,valueMoney:struct<id:string>,valuePeriod:struct<id:string>,valueQuantity:struct<id:string>,valueRange:struct<id:string>,valueReference:struct<id:string>>>,system:string,version:string,code:string,display:string,userSelected:boolean>>,text:string>,system:string,value:string,period:struct<id:string,extension:array<struct<id:string,extension:array<struct<id:string,extension:array<struct<id:string>>,url:string,valueBoolean:boolean,valueCode:string,valueDate:string,valueDateTime:string,valueDecimal:int,valueId:string,valueInteger:int,valuePositiveInt:int,valueString:string,valueTime:string,valueUnsignedInt:int,valueUri:string,valueUrl:string,valueCodeableConcept:struct<id:string>,valueCoding:struct<id:string>,valueCount:struct<id:string>,valueIdentifier:struct<id:string>,valueMoney:struct<id:string>,valuePeriod:struct<id:string>,valueQuantity:struct<id:string>,valueRange:struct<id:string>,valueReference:struct<id:string>>>,url:string,valueBoolean:boolean,valueCode:string,valueDate:string,valueDateTime:string,valueDecimal:int,valueId:string,valueInteger:int,valuePositiveInt:int,valueString:string,valueTime:string,valueUnsignedInt:int,valueUri:string,valueUrl:string,valueCodeableConcept:struct<id:string,extension:array<struct<id:string>>,coding:array<struct<id:string>>,text:string>,valueCoding:struct<id:string,extension:array<struct<id:string>>,system:string,version:string,code:string,display:string,userSelected:boolean>,valueCount:struct<id:string,extension:array<struct<id:string>>,value:int,comparator:string,unit:string,system:string,code:string>,valueIdentifier:struct<id:string,extension:array<struct<id:string>>,use:string,type:struct<id:string>,system:string,value:string,period:struct<id:string>,assigner:struct<id:string>>,valueMoney:struct<id:string,extension:array<struct<id:string>>,value:int,currency:string>,valuePeriod:struct<id:string,extension:array<struct<id:string>>,start:string,end:string>,valueQuantity:struct<id:string,extension:array<struct<id:string>>,value:int,comparator:string,unit:string,system:string,code:string>,valueRange:struct<id:string,extension:array<struct<id:string>>,low:struct<id:string>,high:struct<id:string>>,valueReference:struct<id:string,extension:array<struct<id:string>>,reference:string,type:string,identifier:struct<id:string>,display:string>>>,start:string,end:string>,assigner:struct<id:string,extension:array<struct<id:string,extension:array<struct<id:string,extension:array<struct<id:string>>,url:string,valueBoolean:boolean,valueCode:string,valueDate:string,valueDateTime:string,valueDecimal:int,valueId:string,valueInteger:int,valuePositiveInt:int,valueString:string,valueTime:string,valueUnsignedInt:int,valueUri:string,valueUrl:string,valueCodeableConcept:struct<id:string>,valueCoding:struct<id:string>,valueCount:struct<id:string>,valueIdentifier:struct<id:string>,valueMoney:struct<id:string>,valuePeriod:struct<id:string>,valueQuantity:struct<id:string>,valueRange:struct<id:string>,valueReference:struct<id:string>>>,url:string,valueBoolean:boolean,valueCode:string,valueDate:string,valueDateTime:string,valueDecimal:int,valueId:string,valueInteger:int,valuePositiveInt:int,valueString:string,valueTime:string,valueUnsignedInt:int,valueUri:string,valueUrl:string,valueCodeableConcept:struct<id:string,extension:array<struct<id:string>>,coding:array<struct<id:string>>,text:string>,valueCoding:struct<id:string,extension:array<struct<id:string>>,system:string,version:string,code:string,display:string,userSelected:boolean>,valueCount:struct<id:string,extension:array<struct<id:string>>,value:int,comparator:string,unit:string,system:string,code:string>,valueIdentifier:struct<id:string,extension:array<struct<id:string>>,use:string,type:struct<id:string>,system:string,value:string,period:struct<id:string>,assigner:struct<id:string>>,valueMoney:struct<id:string,extension:array<struct<id:string>>,value:int,currency:string>,valuePeriod:struct<id:string,extension:array<struct<id:string>>,start:string,end:string>,valueQuantity:struct<id:string,extension:array<struct<id:string>>,value:int,comparator:string,unit:string,system:string,code:string>,valueRange:struct<id:string,extension:array<struct<id:string>>,low:struct<id:string>,high:struct<id:string>>,valueReference:struct<id:string,extension:array<struct<id:string>>,reference:string,type:string,identifier:struct<id:string>,display:string>>>,reference:string,type:string,identifier:struct<id:string,extension:array<struct<id:string,extension:array<struct<id:string>>,url:string,valueBoolean:boolean,valueCode:string,valueDate:string,valueDateTime:string,valueDecimal:int,valueId:string,valueInteger:int,valuePositiveInt:int,valueString:string,valueTime:string,valueUnsignedInt:int,valueUri:string,valueUrl:string,valueCodeableConcept:struct<id:string>,valueCoding:struct<id:string>,valueCount:struct<id:string>,valueIdentifier:struct<id:string>,valueMoney:struct<id:string>,valuePeriod:struct<id:string>,valueQuantity:struct<id:string>,valueRange:struct<id:string>,valueReference:struct<id:string>>>,use:string,type:struct<id:string,extension:array<struct<id:string>>,coding:array<struct<id:string>>,text:string>,system:string,value:string,period:struct<id:string,extension:array<struct<id:string>>,start:string,end:string>,assigner:struct<id:string,extension:array<struct<id:string>>,reference:string,type:string,identifier:struct<id:string>,display:string>>,display:string>>>,code:struct<id:string,extension:array<struct<id:string,extension:array<struct<id:string,extension:array<struct<id:string>>,url:string,valueBoolean:boolean,valueCode:string,valueDate:string,valueDateTime:string,valueDecimal:int,valueId:string,valueInteger:int,valuePositiveInt:int,valueString:string,valueTime:string,valueUnsignedInt:int,valueUri:string,valueUrl:string,valueCodeableConcept:struct<id:string,extension:array<struct<id:string>>,coding:array<struct<id:string>>,text:string>,valueCoding:struct<id:string,extension:array<struct<id:string>>,system:string,version:string,code:string,display:string,userSelected:boolean>,valueCount:struct<id:string,extension:array<struct<id:string>>,value:int,comparator:string,unit:string,system:string,code:string>,valueIdentifier:struct<id:string,extension:array<struct<id:string>>,use:string,type:struct<id:string>,system:string,value:string,period:struct<id:string>,assigner:struct<id:string>>,valueMoney:struct<id:string,extension:array<struct<id:string>>,value:int,currency:string>,valuePeriod:struct<id:string,extension:array<struct<id:string>>,start:string,end:string>,valueQuantity:struct<id:string,extension:array<struct<id:string>>,value:int,comparator:string,unit:string,system:string,code:string>,valueRange:struct<id:string,extension:array<struct<id:string>>,low:struct<id:string>,high:struct<id:string>>,valueReference:struct<id:string,extension:array<struct<id:string>>,reference:string,type:string,identifier:struct<id:string>,display:string>>>,url:string,valueBoolean:boolean,valueCode:string,valueDate:string,valueDateTime:string,valueDecimal:int,valueId:string,valueInteger:int,valuePositiveInt:int,valueString:string,valueTime:string,valueUnsignedInt:int,valueUri:string,valueUrl:string,valueCodeableConcept:struct<id:string,extension:array<struct<id:string,extension:array<struct<id:string>>,url:string,valueBoolean:boolean,valueCode:string,valueDate:string,valueDateTime:string,valueDecimal:int,valueId:string,valueInteger:int,valuePositiveInt:int,valueString:string,valueTime:string,valueUnsignedInt:int,valueUri:string,valueUrl:string,valueCodeableConcept:struct<id:string>,valueCoding:struct<id:string>,valueCount:struct<id:string>,valueIdentifier:struct<id:string>,valueMoney:struct<id:string>,valuePeriod:struct<id:string>,valueQuantity:struct<id:string>,valueRange:struct<id:string>,valueReference:struct<id:string>>>,coding:array<struct<id:string,extension:array<struct<id:string>>,system:string,version:string,code:string,display:string,userSelected:boolean>>,text:string>,valueCoding:struct<id:string,extension:array<struct<id:string,extension:array<struct<id:string>>,url:string,valueBoolean:boolean,valueCode:string,valueDate:string,valueDateTime:string,valueDecimal:int,valueId:string,valueInteger:int,valuePositiveInt:int,valueString:string,valueTime:string,valueUnsignedInt:int,valueUri:string,valueUrl:string,valueCodeableConcept:struct<id:string>,valueCoding:struct<id:string>,valueCount:struct<id:string>,valueIdentifier:struct<id:string>,valueMoney:struct<id:string>,valuePeriod:struct<id:string>,valueQuantity:struct<id:string>,valueRange:struct<id:string>,valueReference:struct<id:string>>>,system:string,version:string,code:string,display:string,userSelected:boolean>,valueCount:struct<id:string,extension:array<struct<id:string,extension:array<struct<id:string>>,url:string,valueBoolean:boolean,valueCode:string,valueDate:string,valueDateTime:string,valueDecimal:int,valueId:string,valueInteger:int,valuePositiveInt:int,valueString:string,valueTime:string,valueUnsignedInt:int,valueUri:string,valueUrl:string,valueCodeableConcept:struct<id:string>,valueCoding:struct<id:string>,valueCount:struct<id:string>,valueIdentifier:struct<id:string>,valueMoney:struct<id:string>,valuePeriod:struct<id:string>,valueQuantity:struct<id:string>,valueRange:struct<id:string>,valueReference:struct<id:string>>>,value:int,comparator:string,unit:string,system:string,code:string>,valueIdentifier:struct<id:string,extension:array<struct<id:string,extension:array<struct<id:string>>,url:string,valueBoolean:boolean,valueCode:string,valueDate:string,valueDateTime:string,valueDecimal:int,valueId:string,valueInteger:int,valuePositiveInt:int,valueString:string,valueTime:string,valueUnsignedInt:int,valueUri:string,valueUrl:string,valueCodeableConcept:struct<id:string>,valueCoding:struct<id:string>,valueCount:struct<id:string>,valueIdentifier:struct<id:string>,valueMoney:struct<id:string>,valuePeriod:struct<id:string>,valueQuantity:struct<id:string>,valueRange:struct<id:string>,valueReference:struct<id:string>>>,use:string,type:struct<id:string,extension:array<struct<id:string>>,coding:array<struct<id:string>>,text:string>,system:string,value:string,period:struct<id:string,extension:array<struct<id:string>>,start:string,end:string>,assigner:struct<id:string,extension:array<struct<id:string>>,reference:string,type:string,identifier:struct<id:string>,display:string>>,valueMoney:struct<id:string,extension:array<struct<id:string,extension:array<struct<id:string>>,url:string,valueBoolean:boolean,valueCode:string,valueDate:string,valueDateTime:string,valueDecimal:int,valueId:string,valueInteger:int,valuePositiveInt:int,valueString:string,valueTime:string,valueUnsignedInt:int,valueUri:string,valueUrl:string,valueCodeableConcept:struct<id:string>,valueCoding:struct<id:string>,valueCount:struct<id:string>,valueIdentifier:struct<id:string>,valueMoney:struct<id:string>,valuePeriod:struct<id:string>,valueQuantity:struct<id:string>,valueRange:struct<id:string>,valueReference:struct<id:string>>>,value:int,currency:string>,valuePeriod:struct<id:string,extension:array<struct<id:string,extension:array<struct<id:string>>,url:string,valueBoolean:boolean,valueCode:string,valueDate:string,valueDateTime:string,valueDecimal:int,valueId:string,valueInteger:int,valuePositiveInt:int,valueString:string,valueTime:string,valueUnsignedInt:int,valueUri:string,valueUrl:string,valueCodeableConcept:struct<id:string>,valueCoding:struct<id:string>,valueCount:struct<id:string>,valueIdentifier:struct<id:string>,valueMoney:struct<id:string>,valuePeriod:struct<id:string>,valueQuantity:struct<id:string>,valueRange:struct<id:string>,valueReference:struct<id:string>>>,start:string,end:string>,valueQuantity:struct<id:string,extension:array<struct<id:string,extension:array<struct<id:string>>,url:string,valueBoolean:boolean,valueCode:string,valueDate:string,valueDateTime:string,valueDecimal:int,valueId:string,valueInteger:int,valuePositiveInt:int,valueString:string,valueTime:string,valueUnsignedInt:int,valueUri:string,valueUrl:string,valueCodeableConcept:struct<id:string>,valueCoding:struct<id:string>,valueCount:struct<id:string>,valueIdentifier:struct<id:string>,valueMoney:struct<id:string>,valuePeriod:struct<id:string>,valueQuantity:struct<id:string>,valueRange:struct<id:string>,valueReference:struct<id:string>>>,value:int,comparator:string,unit:string,system:string,code:string>,valueRange:struct<id:string,extension:array<struct<id:string,extension:array<struct<id:string>>,url:string,valueBoolean:boolean,valueCode:string,valueDate:string,valueDateTime:string,valueDecimal:int,valueId:string,valueInteger:int,valuePositiveInt:int,valueString:string,valueTime:string,valueUnsignedInt:int,valueUri:string,valueUrl:string,valueCodeableConcept:struct<id:string>,valueCoding:struct<id:string>,valueCount:struct<id:string>,valueIdentifier:struct<id:string>,valueMoney:struct<id:string>,valuePeriod:struct<id:string>,valueQuantity:struct<id:string>,valueRange:struct<id:string>,valueReference:struct<id:string>>>,low:struct<id:string,extension:array<struct<id:string>>,value:int,comparator:string,unit:string,system:string,code:string>,high:struct<id:string,extension:array<struct<id:string>>,value:int,comparator:string,unit:string,system:string,code:string>>,valueReference:struct<id:string,extension:array<struct<id:string,extension:array<struct<id:string>>,url:string,valueBoolean:boolean,valueCode:string,valueDate:string,valueDateTime:string,valueDecimal:int,valueId:string,valueInteger:int,valuePositiveInt:int,valueString:string,valueTime:string,valueUnsignedInt:int,valueUri:string,valueUrl:string,valueCodeableConcept:struct<id:string>,valueCoding:struct<id:string>,valueCount:struct<id:string>,valueIdentifier:struct<id:string>,valueMoney:struct<id:string>,valuePeriod:struct<id:string>,valueQuantity:struct<id:string>,valueRange:struct<id:string>,valueReference:struct<id:string>>>,reference:string,type:string,identifier:struct<id:string,extension:array<struct<id:string>>,use:string,type:struct<id:string>,system:string,value:string,period:struct<id:string>,assigner:struct<id:string>>,display:string>>>,coding:array<struct<id:string,extension:array<struct<id:string,extension:array<struct<id:string,extension:array<struct<id:string>>,url:string,valueBoolean:boolean,valueCode:string,valueDate:string,valueDateTime:string,valueDecimal:int,valueId:string,valueInteger:int,valuePositiveInt:int,valueString:string,valueTime:string,valueUnsignedInt:int,valueUri:string,valueUrl:string,valueCodeableConcept:struct<id:string>,valueCoding:struct<id:string>,valueCount:struct<id:string>,valueIdentifier:struct<id:string>,valueMoney:struct<id:string>,valuePeriod:struct<id:string>,valueQuantity:struct<id:string>,valueRange:struct<id:string>,valueReference:struct<id:string>>>,url:string,valueBoolean:boolean,valueCode:string,valueDate:string,valueDateTime:string,valueDecimal:int,valueId:string,valueInteger:int,valuePositiveInt:int,valueString:string,valueTime:string,valueUnsignedInt:int,valueUri:string,valueUrl:string,valueCodeableConcept:struct<id:string,extension:array<struct<id:string>>,coding:array<struct<id:string>>,text:string>,valueCoding:struct<id:string,extension:array<struct<id:string>>,system:string,version:string,code:string,display:string,userSelected:boolean>,valueCount:struct<id:string,extension:array<struct<id:string>>,value:int,comparator:string,unit:string,system:string,code:string>,valueIdentifier:struct<id:string,extension:array<struct<id:string>>,use:string,type:struct<id:string>,system:string,value:string,period:struct<id:string>,assigner:struct<id:string>>,valueMoney:struct<id:string,extension:array<struct<id:string>>,value:int,currency:string>,valuePeriod:struct<id:string,extension:array<struct<id:string>>,start:string,end:string>,valueQuantity:struct<id:string,extension:array<struct<id:string>>,value:int,comparator:string,unit:string,system:string,code:string>,valueRange:struct<id:string,extension:array<struct<id:string>>,low:struct<id:string>,high:struct<id:string>>,valueReference:struct<id:string,extension:array<struct<id:string>>,reference:string,type:string,identifier:struct<id:string>,display:string>>>,system:string,version:string,code:string,display:string,userSelected:boolean>>,text:string>,period:struct<id:string,extension:array<struct<id:string,extension:array<struct<id:string,extension:array<struct<id:string>>,url:string,valueBoolean:boolean,valueCode:string,valueDate:string,valueDateTime:string,valueDecimal:int,valueId:string,valueInteger:int,valuePositiveInt:int,valueString:string,valueTime:string,valueUnsignedInt:int,valueUri:string,valueUrl:string,valueCodeableConcept:struct<id:string,extension:array<struct<id:string>>,coding:array<struct<id:string>>,text:string>,valueCoding:struct<id:string,extension:array<struct<id:string>>,system:string,version:string,code:string,display:string,userSelected:boolean>,valueCount:struct<id:string,extension:array<struct<id:string>>,value:int,comparator:string,unit:string,system:string,code:string>,valueIdentifier:struct<id:string,extension:array<struct<id:string>>,use:string,type:struct<id:string>,system:string,value:string,period:struct<id:string>,assigner:struct<id:string>>,valueMoney:struct<id:string,extension:array<struct<id:string>>,value:int,currency:string>,valuePeriod:struct<id:string,extension:array<struct<id:string>>,start:string,end:string>,valueQuantity:struct<id:string,extension:array<struct<id:string>>,value:int,comparator:string,unit:string,system:string,code:string>,valueRange:struct<id:string,extension:array<struct<id:string>>,low:struct<id:string>,high:struct<id:string>>,valueReference:struct<id:string,extension:array<struct<id:string>>,reference:string,type:string,identifier:struct<id:string>,display:string>>>,url:string,valueBoolean:boolean,valueCode:string,valueDate:string,valueDateTime:string,valueDecimal:int,valueId:string,valueInteger:int,valuePositiveInt:int,valueString:string,valueTime:string,valueUnsignedInt:int,valueUri:string,valueUrl:string,valueCodeableConcept:struct<id:string,extension:array<struct<id:string,extension:array<struct<id:string>>,url:string,valueBoolean:boolean,valueCode:string,valueDate:string,valueDateTime:string,valueDecimal:int,valueId:string,valueInteger:int,valuePositiveInt:int,valueString:string,valueTime:string,valueUnsignedInt:int,valueUri:string,valueUrl:string,valueCodeableConcept:struct<id:string>,valueCoding:struct<id:string>,valueCount:struct<id:string>,valueIdentifier:struct<id:string>,valueMoney:struct<id:string>,valuePeriod:struct<id:string>,valueQuantity:struct<id:string>,valueRange:struct<id:string>,valueReference:struct<id:string>>>,coding:array<struct<id:string,extension:array<struct<id:string>>,system:string,version:string,code:string,display:string,userSelected:boolean>>,text:string>,valueCoding:struct<id:string,extension:array<struct<id:string,extension:array<struct<id:string>>,url:string,valueBoolean:boolean,valueCode:string,valueDate:string,valueDateTime:string,valueDecimal:int,valueId:string,valueInteger:int,valuePositiveInt:int,valueString:string,valueTime:string,valueUnsignedInt:int,valueUri:string,valueUrl:string,valueCodeableConcept:struct<id:string>,valueCoding:struct<id:string>,valueCount:struct<id:string>,valueIdentifier:struct<id:string>,valueMoney:struct<id:string>,valuePeriod:struct<id:string>,valueQuantity:struct<id:string>,valueRange:struct<id:string>,valueReference:struct<id:string>>>,system:string,version:string,code:string,display:string,userSelected:boolean>,valueCount:struct<id:string,extension:array<struct<id:string,extension:array<struct<id:string>>,url:string,valueBoolean:boolean,valueCode:string,valueDate:string,valueDateTime:string,valueDecimal:int,valueId:string,valueInteger:int,valuePositiveInt:int,valueString:string,valueTime:string,valueUnsignedInt:int,valueUri:string,valueUrl:string,valueCodeableConcept:struct<id:string>,valueCoding:struct<id:string>,valueCount:struct<id:string>,valueIdentifier:struct<id:string>,valueMoney:struct<id:string>,valuePeriod:struct<id:string>,valueQuantity:struct<id:string>,valueRange:struct<id:string>,valueReference:struct<id:string>>>,value:int,comparator:string,unit:string,system:string,code:string>,valueIdentifier:struct<id:string,extension:array<struct<id:string,extension:array<struct<id:string>>,url:string,valueBoolean:boolean,valueCode:string,valueDate:string,valueDateTime:string,valueDecimal:int,valueId:string,valueInteger:int,valuePositiveInt:int,valueString:string,valueTime:string,valueUnsignedInt:int,valueUri:string,valueUrl:string,valueCodeableConcept:struct<id:string>,valueCoding:struct<id:string>,valueCount:struct<id:string>,valueIdentifier:struct<id:string>,valueMoney:struct<id:string>,valuePeriod:struct<id:string>,valueQuantity:struct<id:string>,valueRange:struct<id:string>,valueReference:struct<id:string>>>,use:string,type:struct<id:string,extension:array<struct<id:string>>,coding:array<struct<id:string>>,text:string>,system:string,value:string,period:struct<id:string,extension:array<struct<id:string>>,start:string,end:string>,assigner:struct<id:string,extension:array<struct<id:string>>,reference:string,type:string,identifier:struct<id:string>,display:string>>,valueMoney:struct<id:string,extension:array<struct<id:string,extension:array<struct<id:string>>,url:string,valueBoolean:boolean,valueCode:string,valueDate:string,valueDateTime:string,valueDecimal:int,valueId:string,valueInteger:int,valuePositiveInt:int,valueString:string,valueTime:string,valueUnsignedInt:int,valueUri:string,valueUrl:string,valueCodeableConcept:struct<id:string>,valueCoding:struct<id:string>,valueCount:struct<id:string>,valueIdentifier:struct<id:string>,valueMoney:struct<id:string>,valuePeriod:struct<id:string>,valueQuantity:struct<id:string>,valueRange:struct<id:string>,valueReference:struct<id:string>>>,value:int,currency:string>,valuePeriod:struct<id:string,extension:array<struct<id:string,extension:array<struct<id:string>>,url:string,valueBoolean:boolean,valueCode:string,valueDate:string,valueDateTime:string,valueDecimal:int,valueId:string,valueInteger:int,valuePositiveInt:int,valueString:string,valueTime:string,valueUnsignedInt:int,valueUri:string,valueUrl:string,valueCodeableConcept:struct<id:string>,valueCoding:struct<id:string>,valueCount:struct<id:string>,valueIdentifier:struct<id:string>,valueMoney:struct<id:string>,valuePeriod:struct<id:string>,valueQuantity:struct<id:string>,valueRange:struct<id:string>,valueReference:struct<id:string>>>,start:string,end:string>,valueQuantity:struct<id:string,extension:array<struct<id:string,extension:array<struct<id:string>>,url:string,valueBoolean:boolean,valueCode:string,valueDate:string,valueDateTime:string,valueDecimal:int,valueId:string,valueInteger:int,valuePositiveInt:int,valueString:string,valueTime:string,valueUnsignedInt:int,valueUri:string,valueUrl:string,valueCodeableConcept:struct<id:string>,valueCoding:struct<id:string>,valueCount:struct<id:string>,valueIdentifier:struct<id:string>,valueMoney:struct<id:string>,valuePeriod:struct<id:string>,valueQuantity:struct<id:string>,valueRange:struct<id:string>,valueReference:struct<id:string>>>,value:int,comparator:string,unit:string,system:string,code:string>,valueRange:struct<id:string,extension:array<struct<id:string,extension:array<struct<id:string>>,url:string,valueBoolean:boolean,valueCode:string,valueDate:string,valueDateTime:string,valueDecimal:int,valueId:string,valueInteger:int,valuePositiveInt:int,valueString:string,valueTime:string,valueUnsignedInt:int,valueUri:string,valueUrl:string,valueCodeableConcept:struct<id:string>,valueCoding:struct<id:string>,valueCount:struct<id:string>,valueIdentifier:struct<id:string>,valueMoney:struct<id:string>,valuePeriod:struct<id:string>,valueQuantity:struct<id:string>,valueRange:struct<id:string>,valueReference:struct<id:string>>>,low:struct<id:string,extension:array<struct<id:string>>,value:int,comparator:string,unit:string,system:string,code:string>,high:struct<id:string,extension:array<struct<id:string>>,value:int,comparator:string,unit:string,system:string,code:string>>,valueReference:struct<id:string,extension:array<struct<id:string,extension:array<struct<id:string>>,url:string,valueBoolean:boolean,valueCode:string,valueDate:string,valueDateTime:string,valueDecimal:int,valueId:string,valueInteger:int,valuePositiveInt:int,valueString:string,valueTime:string,valueUnsignedInt:int,valueUri:string,valueUrl:string,valueCodeableConcept:struct<id:string>,valueCoding:struct<id:string>,valueCount:struct<id:string>,valueIdentifier:struct<id:string>,valueMoney:struct<id:string>,valuePeriod:struct<id:string>,valueQuantity:struct<id:string>,valueRange:struct<id:string>,valueReference:struct<id:string>>>,reference:string,type:string,identifier:struct<id:string,extension:array<struct<id:string>>,use:string,type:struct<id:string>,system:string,value:string,period:struct<id:string>,assigner:struct<id:string>>,display:string>>>,start:string,end:string>,issuer:struct<id:string,extension:array<struct<id:string,extension:array<struct<id:string,extension:array<struct<id:string>>,url:string,valueBoolean:boolean,valueCode:string,valueDate:string,valueDateTime:string,valueDecimal:int,valueId:string,valueInteger:int,valuePositiveInt:int,valueString:string,valueTime:string,valueUnsignedInt:int,valueUri:string,valueUrl:string,valueCodeableConcept:struct<id:string,extension:array<struct<id:string>>,coding:array<struct<id:string>>,text:string>,valueCoding:struct<id:string,extension:array<struct<id:string>>,system:string,version:string,code:string,display:string,userSelected:boolean>,valueCount:struct<id:string,extension:array<struct<id:string>>,value:int,comparator:string,unit:string,system:string,code:string>,valueIdentifier:struct<id:string,extension:array<struct<id:string>>,use:string,type:struct<id:string>,system:string,value:string,period:struct<id:string>,assigner:struct<id:string>>,valueMoney:struct<id:string,extension:array<struct<id:string>>,value:int,currency:string>,valuePeriod:struct<id:string,extension:array<struct<id:string>>,start:string,end:string>,valueQuantity:struct<id:string,extension:array<struct<id:string>>,value:int,comparator:string,unit:string,system:string,code:string>,valueRange:struct<id:string,extension:array<struct<id:string>>,low:struct<id:string>,high:struct<id:string>>,valueReference:struct<id:string,extension:array<struct<id:string>>,reference:string,type:string,identifier:struct<id:string>,display:string>>>,url:string,valueBoolean:boolean,valueCode:string,valueDate:string,valueDateTime:string,valueDecimal:int,valueId:string,valueInteger:int,valuePositiveInt:int,valueString:string,valueTime:string,valueUnsignedInt:int,valueUri:string,valueUrl:string,valueCodeableConcept:struct<id:string,extension:array<struct<id:string,extension:array<struct<id:string>>,url:string,valueBoolean:boolean,valueCode:string,valueDate:string,valueDateTime:string,valueDecimal:int,valueId:string,valueInteger:int,valuePositiveInt:int,valueString:string,valueTime:string,valueUnsignedInt:int,valueUri:string,valueUrl:string,valueCodeableConcept:struct<id:string>,valueCoding:struct<id:string>,valueCount:struct<id:string>,valueIdentifier:struct<id:string>,valueMoney:struct<id:string>,valuePeriod:struct<id:string>,valueQuantity:struct<id:string>,valueRange:struct<id:string>,valueReference:struct<id:string>>>,coding:array<struct<id:string,extension:array<struct<id:string>>,system:string,version:string,code:string,display:string,userSelected:boolean>>,text:string>,valueCoding:struct<id:string,extension:array<struct<id:string,extension:array<struct<id:string>>,url:string,valueBoolean:boolean,valueCode:string,valueDate:string,valueDateTime:string,valueDecimal:int,valueId:string,valueInteger:int,valuePositiveInt:int,valueString:string,valueTime:string,valueUnsignedInt:int,valueUri:string,valueUrl:string,valueCodeableConcept:struct<id:string>,valueCoding:struct<id:string>,valueCount:struct<id:string>,valueIdentifier:struct<id:string>,valueMoney:struct<id:string>,valuePeriod:struct<id:string>,valueQuantity:struct<id:string>,valueRange:struct<id:string>,valueReference:struct<id:string>>>,system:string,version:string,code:string,display:string,userSelected:boolean>,valueCount:struct<id:string,extension:array<struct<id:string,extension:array<struct<id:string>>,url:string,valueBoolean:boolean,valueCode:string,valueDate:string,valueDateTime:string,valueDecimal:int,valueId:string,valueInteger:int,valuePositiveInt:int,valueString:string,valueTime:string,valueUnsignedInt:int,valueUri:string,valueUrl:string,valueCodeableConcept:struct<id:string>,valueCoding:struct<id:string>,valueCount:struct<id:string>,valueIdentifier:struct<id:string>,valueMoney:struct<id:string>,valuePeriod:struct<id:string>,valueQuantity:struct<id:string>,valueRange:struct<id:string>,valueReference:struct<id:string>>>,value:int,comparator:string,unit:string,system:string,code:string>,valueIdentifier:struct<id:string,extension:array<struct<id:string,extension:array<struct<id:string>>,url:string,valueBoolean:boolean,valueCode:string,valueDate:string,valueDateTime:string,valueDecimal:int,valueId:string,valueInteger:int,valuePositiveInt:int,valueString:string,valueTime:string,valueUnsignedInt:int,valueUri:string,valueUrl:string,valueCodeableConcept:struct<id:string>,valueCoding:struct<id:string>,valueCount:struct<id:string>,valueIdentifier:struct<id:string>,valueMoney:struct<id:string>,valuePeriod:struct<id:string>,valueQuantity:struct<id:string>,valueRange:struct<id:string>,valueReference:struct<id:string>>>,use:string,type:struct<id:string,extension:array<struct<id:string>>,coding:array<struct<id:string>>,text:string>,system:string,value:string,period:struct<id:string,extension:array<struct<id:string>>,start:string,end:string>,assigner:struct<id:string,extension:array<struct<id:string>>,reference:string,type:string,identifier:struct<id:string>,display:string>>,valueMoney:struct<id:string,extension:array<struct<id:string,extension:array<struct<id:string>>,url:string,valueBoolean:boolean,valueCode:string,valueDate:string,valueDateTime:string,valueDecimal:int,valueId:string,valueInteger:int,valuePositiveInt:int,valueString:string,valueTime:string,valueUnsignedInt:int,valueUri:string,valueUrl:string,valueCodeableConcept:struct<id:string>,valueCoding:struct<id:string>,valueCount:struct<id:string>,valueIdentifier:struct<id:string>,valueMoney:struct<id:string>,valuePeriod:struct<id:string>,valueQuantity:struct<id:string>,valueRange:struct<id:string>,valueReference:struct<id:string>>>,value:int,currency:string>,valuePeriod:struct<id:string,extension:array<struct<id:string,extension:array<struct<id:string>>,url:string,valueBoolean:boolean,valueCode:string,valueDate:string,valueDateTime:string,valueDecimal:int,valueId:string,valueInteger:int,valuePositiveInt:int,valueString:string,valueTime:string,valueUnsignedInt:int,valueUri:string,valueUrl:string,valueCodeableConcept:struct<id:string>,valueCoding:struct<id:string>,valueCount:struct<id:string>,valueIdentifier:struct<id:string>,valueMoney:struct<id:string>,valuePeriod:struct<id:string>,valueQuantity:struct<id:string>,valueRange:struct<id:string>,valueReference:struct<id:string>>>,start:string,end:string>,valueQuantity:struct<id:string,extension:array<struct<id:string,extension:array<struct<id:string>>,url:string,valueBoolean:boolean,valueCode:string,valueDate:string,valueDateTime:string,valueDecimal:int,valueId:string,valueInteger:int,valuePositiveInt:int,valueString:string,valueTime:string,valueUnsignedInt:int,valueUri:string,valueUrl:string,valueCodeableConcept:struct<id:string>,valueCoding:struct<id:string>,valueCount:struct<id:string>,valueIdentifier:struct<id:string>,valueMoney:struct<id:string>,valuePeriod:struct<id:string>,valueQuantity:struct<id:string>,valueRange:struct<id:string>,valueReference:struct<id:string>>>,value:int,comparator:string,unit:string,system:string,code:string>,valueRange:struct<id:string,extension:array<struct<id:string,extension:array<struct<id:string>>,url:string,valueBoolean:boolean,valueCode:string,valueDate:string,valueDateTime:string,valueDecimal:int,valueId:string,valueInteger:int,valuePositiveInt:int,valueString:string,valueTime:string,valueUnsignedInt:int,valueUri:string,valueUrl:string,valueCodeableConcept:struct<id:string>,valueCoding:struct<id:string>,valueCount:struct<id:string>,valueIdentifier:struct<id:string>,valueMoney:struct<id:string>,valuePeriod:struct<id:string>,valueQuantity:struct<id:string>,valueRange:struct<id:string>,valueReference:struct<id:string>>>,low:struct<id:string,extension:array<struct<id:string>>,value:int,comparator:string,unit:string,system:string,code:string>,high:struct<id:string,extension:array<struct<id:string>>,value:int,comparator:string,unit:string,system:string,code:string>>,valueReference:struct<id:string,extension:array<struct<id:string,extension:array<struct<id:string>>,url:string,valueBoolean:boolean,valueCode:string,valueDate:string,valueDateTime:string,valueDecimal:int,valueId:string,valueInteger:int,valuePositiveInt:int,valueString:string,valueTime:string,valueUnsignedInt:int,valueUri:string,valueUrl:string,valueCodeableConcept:struct<id:string>,valueCoding:struct<id:string>,valueCount:struct<id:string>,valueIdentifier:struct<id:string>,valueMoney:struct<id:string>,valuePeriod:struct<id:string>,valueQuantity:struct<id:string>,valueRange:struct<id:string>,valueReference:struct<id:string>>>,reference:string,type:string,identifier:struct<id:string,extension:array<struct<id:string>>,use:string,type:struct<id:string>,system:string,value:string,period:struct<id:string>,assigner:struct<id:string>>,display:string>>>,reference:string,type:string,identifier:struct<id:string,extension:array<struct<id:string,extension:array<struct<id:string,extension:array<struct<id:string>>,url:string,valueBoolean:boolean,valueCode:string,valueDate:string,valueDateTime:string,valueDecimal:int,valueId:string,valueInteger:int,valuePositiveInt:int,valueString:string,valueTime:string,valueUnsignedInt:int,valueUri:string,valueUrl:string,valueCodeableConcept:struct<id:string>,valueCoding:struct<id:string>,valueCount:struct<id:string>,valueIdentifier:struct<id:string>,valueMoney:struct<id:string>,valuePeriod:struct<id:string>,valueQuantity:struct<id:string>,valueRange:struct<id:string>,valueReference:struct<id:string>>>,url:string,valueBoolean:boolean,valueCode:string,valueDate:string,valueDateTime:string,valueDecimal:int,valueId:string,valueInteger:int,valuePositiveInt:int,valueString:string,valueTime:string,valueUnsignedInt:int,valueUri:string,valueUrl:string,valueCodeableConcept:struct<id:string,extension:array<struct<id:string>>,coding:array<struct<id:string>>,text:string>,valueCoding:struct<id:string,extension:array<struct<id:string>>,system:string,version:string,code:string,display:string,userSelected:boolean>,valueCount:struct<id:string,extension:array<struct<id:string>>,value:int,comparator:string,unit:string,system:string,code:string>,valueIdentifier:struct<id:string,extension:array<struct<id:string>>,use:string,type:struct<id:string>,system:string,value:string,period:struct<id:string>,assigner:struct<id:string>>,valueMoney:struct<id:string,extension:array<struct<id:string>>,value:int,currency:string>,valuePeriod:struct<id:string,extension:array<struct<id:string>>,start:string,end:string>,valueQuantity:struct<id:string,extension:array<struct<id:string>>,value:int,comparator:string,unit:string,system:string,code:string>,valueRange:struct<id:string,extension:array<struct<id:string>>,low:struct<id:string>,high:struct<id:string>>,valueReference:struct<id:string,extension:array<struct<id:string>>,reference:string,type:string,identifier:struct<id:string>,display:string>>>,use:string,type:struct<id:string,extension:array<struct<id:string,extension:array<struct<id:string>>,url:string,valueBoolean:boolean,valueCode:string,valueDate:string,valueDateTime:string,valueDecimal:int,valueId:string,valueInteger:int,valuePositiveInt:int,valueString:string,valueTime:string,valueUnsignedInt:int,valueUri:string,valueUrl:string,valueCodeableConcept:struct<id:string>,valueCoding:struct<id:string>,valueCount:struct<id:string>,valueIdentifier:struct<id:string>,valueMoney:struct<id:string>,valuePeriod:struct<id:string>,valueQuantity:struct<id:string>,valueRange:struct<id:string>,valueReference:struct<id:string>>>,coding:array<struct<id:string,extension:array<struct<id:string>>,system:string,version:string,code:string,display:string,userSelected:boolean>>,text:string>,system:string,value:string,period:struct<id:string,extension:array<struct<id:string,extension:array<struct<id:string>>,url:string,valueBoolean:boolean,valueCode:string,valueDate:string,valueDateTime:string,valueDecimal:int,valueId:string,valueInteger:int,valuePositiveInt:int,valueString:string,valueTime:string,valueUnsignedInt:int,valueUri:string,valueUrl:string,valueCodeableConcept:struct<id:string>,valueCoding:struct<id:string>,valueCount:struct<id:string>,valueIdentifier:struct<id:string>,valueMoney:struct<id:string>,valuePeriod:struct<id:string>,valueQuantity:struct<id:string>,valueRange:struct<id:string>,valueReference:struct<id:string>>>,start:string,end:string>,assigner:struct<id:string,extension:array<struct<id:string,extension:array<struct<id:string>>,url:string,valueBoolean:boolean,valueCode:string,valueDate:string,valueDateTime:string,valueDecimal:int,valueId:string,valueInteger:int,valuePositiveInt:int,valueString:string,valueTime:string,valueUnsignedInt:int,valueUri:string,valueUrl:string,valueCodeableConcept:struct<id:string>,valueCoding:struct<id:string>,valueCount:struct<id:string>,valueIdentifier:struct<id:string>,valueMoney:struct<id:string>,valuePeriod:struct<id:string>,valueQuantity:struct<id:string>,valueRange:struct<id:string>,valueReference:struct<id:string>>>,reference:string,type:string,identifier:struct<id:string,extension:array<struct<id:string>>,use:string,type:struct<id:string>,system:string,value:string,period:struct<id:string>,assigner:struct<id:string>>,display:string>>,display:string>>>"
    return [parse_struct(struct1), parse_struct(struct2)]


def parse_struct(struct1: str) -> str:
    remaining_text = struct1.replace("'", "")
    stack = []
    formatted_struct1 = ""
    last_item_was_letter = False
    while remaining_text:
        if remaining_text.startswith("array<"):
            remaining_text = remaining_text[6:]
            formatted_struct1 += "["
            stack.append("[")
            last_item_was_letter = False
        elif remaining_text.startswith("struct<"):
            remaining_text = remaining_text[7:]
            formatted_struct1 += "{"
            stack.append("{")
            last_item_was_letter = False
        elif remaining_text.startswith(">"):
            remaining_text = remaining_text[1:]
            if last_item_was_letter:
                formatted_struct1 += '"'
            if len(stack) > 0:
                pop = stack.pop()
                if pop == "[":
                    formatted_struct1 += "]"
                else:
                    formatted_struct1 += "}"
            last_item_was_letter = False
        else:
            item = remaining_text[0]
            remaining_text = remaining_text[1:]
            if item == ":":
                if last_item_was_letter:
                    formatted_struct1 += '"'
                formatted_struct1 += ":"
                last_item_was_letter = False
            elif item == ",":
                if last_item_was_letter:
                    formatted_struct1 += '"'
                formatted_struct1 += ","
                last_item_was_letter = False
            else:
                if not last_item_was_letter:
                    formatted_struct1 += '"'
                last_item_was_letter = True
                formatted_struct1 += item
    return formatted_struct1



from typing import Dict, Any

from library.features.carriers_python.v1.features_carriers_python_v1 import (
    FeaturesCarriersPythonV1,
)

from library.features.carriers.v1.features_carriers_v1 import FeaturesCarriersV1

from spark_pipeline_framework.progress_logger.progress_logger import ProgressLogger

from spark_pipeline_framework.pipelines.framework_pipeline import FrameworkPipeline
from spark_pipeline_framework.transformers.framework_csv_loader.v1.framework_csv_loader import (
    FrameworkCsvLoader,
)


class MyPipeline(FrameworkPipeline):
    def __init__(self, parameters: Dict[str, Any], progress_logger: ProgressLogger):
        super(MyPipeline, self).__init__(
            parameters=parameters, progress_logger=progress_logger
        )
        self.transformers = self.create_steps(
            [
                FrameworkCsvLoader(
                    view="flights",
                    path_to_csv=parameters["flights_path"],
                    parameters=parameters,
                    progress_logger=progress_logger,
                ),
                FeaturesCarriersV1(
                    parameters=parameters, progress_logger=progress_logger
                ).transformers,
                FeaturesCarriersPythonV1(
                    parameters=parameters, progress_logger=progress_logger
                ).transformers,
            ]
        )
